/* diagram.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include "main.h"
#include "header.h"
#include "gui.h"
#include "diagram.h"
#include "network.h"
#include "interface.h"
#include "device.h"
#include "list.h"
#include "data.h"
#include "lcomputer.xpm"
#include "lserver.xpm"
#include "lprinter.xpm"
#include "lswitch.xpm"
#include "lrouter.xpm"
#include "lbrouter.xpm"
#include "lcloud.xpm"
#include "lunknown.xpm"

Diagram::Diagram(GtkWidget *w, Main *m)
{
	main = m;
	window = w;

	alloc_color(red, 65535, 0, 0);
	alloc_color(green, 0, 65535, 0);
	alloc_color(blue, 0, 0, 65535);

	alloc_color(darkred, 32767, 0, 0);
	alloc_color(darkgreen, 0, 32767, 0);
	alloc_color(darkblue, 0, 0, 32767);

	alloc_color(yellow, 65535, 65535, 0);
	alloc_color(cyan, 0, 65535, 65535);
	alloc_color(magenta, 65535, 0, 65535);

	alloc_color(darkyellow, 32767, 32767, 0);
	alloc_color(darkcyan, 0, 32767, 32767);
	alloc_color(darkmagenta, 32767, 0, 32767);

	alloc_color(lightgray, 49151, 49151, 49151);
	alloc_color(gray, 32767, 32767, 32767);
	alloc_color(darkgray, 16383, 16383, 16383);
	alloc_color(bggray, 62343, 63343, 64343);

	alloc_color(black, 0, 0, 0);
	alloc_color(white, 65535, 65535, 65535);
	alloc_color(orange, 65535, 32767, 32767);
	alloc_color(purple, 32767, 0, 65535);

	gc = gdk_gc_new(window->window);
	gc2 = gdk_gc_new(window->window);
	gc3 = gdk_gc_new(window->window);
	points = new GdkPoint[4];

	lserver.load(w, lserver_xpm);
	lmachine.load(w, lcomputer_xpm);
	lswitch.load(w, lswitch_xpm);
	lrouter.load(w, lrouter_xpm);
	lbrouter.load(w, lbrouter_xpm);
	lprinter.load(w, lprinter_xpm);
	lcloud.load(w, lcloud_xpm);
	lunknown.load(w, lunknown_xpm);
}

Diagram::~Diagram()
{
	if (gc) gdk_gc_unref(gc);
	if (gc2) gdk_gc_unref(gc2);
	if (gc3) gdk_gc_unref(gc3);
	delete points;
}

void Diagram::scale_images(float x, float y)
{
	lserver.scale(int(96/x), int(96/y));
	lmachine.scale(int(96/x), int(96/y));
	lswitch.scale(int(144/x), int(144/y));
	lrouter.scale(int(144/x), int(144/y));
	lprinter.scale(int(96/x), int(96/y));
	lcloud.scale(int(96/x), int(96/y));
	lunknown.scale(int(144/x), int(144/y));
	lbrouter.scale(int(144/x), int(144/y));
}

void Diagram::interface_color(GdkGC *gc, int type)
{
	switch (type)
	{
		case 6:  gdk_gc_set_foreground(gc, &darkgreen); break;
		case 15: gdk_gc_set_foreground(gc, &blue); break;
		case 18:
		case 22:
		case 23:
		case 28: gdk_gc_set_foreground(gc, &darkred); break;
		case 37:
		case 39:
		case 49:
		case 53:
		case 59: gdk_gc_set_foreground(gc, &yellow); break;
		case 1:
		case 24:
		default: gdk_gc_set_foreground(gc, &darkblue);
	}
}

void Diagram::draw(GdkPixmap *canvas, Data *data, float scale, long xp, long yp, int w, int h, int pp)
{
	float tx, ty, fntsz;
	Network *n;
	Device *r;
	Machine *m;
	Link *ln, *ln2, *ln3;
	long width, height;
	Interface *in;
	GdkFont *fnt, *fnt2;
	char fntstr[256], buf[64];
	int i, j, ww, labelx, labely, cnt1, cnt2;

	if (data==NULL) return;

	fntsz = 128/scale;	// 112 is the MAX point size when fully zoomed in
	sprintf(fntstr, "-*-helvetica-medium-r-normal-*-*-%d0-*-*-p-*-iso8859-1",
		(int)fntsz);
	fnt = gdk_font_load(fntstr);

	fntsz = 56/scale;	// half the size
	sprintf(fntstr, "-*-helvetica-medium-r-normal-*-*-%d0-*-*-p-*-iso8859-1",
		(int)fntsz);
	fnt2 = gdk_font_load(fntstr);

	width = w;
	height = h;

// first we need to scale everything to fit

	scale_images(scale, scale);
	tx = xp/scale; ty=yp/scale;
	for (ln=data->devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		r->rx = (r->x/scale)+(width/2)-tx;
		r->ry = (r->y/scale)+(height/2)-ty;
	}
	for (ln=data->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		n->rx = (n->x/scale)+(width/2)-tx;
		n->ry = (n->y/scale)+(height/2)-ty;
	}
	for (ln=data->machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		m->rx = (m->x/scale)+(width/2)-tx;
		m->ry = (m->y/scale)+(height/2)-ty;
	}

//	gdk_gc_set_foreground(gc, &red);
//	gdk_gc_set_line_attributes(gc, 3, GDK_LINE_ON_OFF_DASH,
//		GDK_CAP_BUTT, GDK_JOIN_MITER);


	// clear window
	if (pp==0)	// pp = print preview
	{
		gdk_gc_set_foreground(gc, &background_color);
		gdk_draw_rectangle(canvas, gc, TRUE, 0, 0, w, h);
	}

	// line width == 6 when zoomed in & 1 when scale == 6
	gdk_gc_set_line_attributes(gc, int(12/scale), GDK_LINE_SOLID,
		GDK_CAP_BUTT, GDK_JOIN_MITER);

	// work out what we need to draw
	for (ln=data->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		cnt1 = 0;
		cnt2 = 0;
		for (ln2=n->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			in = (Interface*) ln2->item;
			if (in)
			{
				if ((in->type==15) && (n->subtype==network_auto))
					n->subtype = network_ring; // fddi
				if (in->device)
				{
					if (in->device->shown==TRUE) cnt2++;
					cnt1 += in->machines.size();
				}
			}
		}
		if ((n->subtype==network_ring) && (cnt2<2)) n->subtype = network_cloud;
		if ((n->subtype==network_auto) && (cnt2==2) && (cnt1==0))
			n->subtype=network_p2p;
		if (n->subtype==network_auto) n->subtype = network_cloud;		//default
		if (n->subtype==network_ring)
		{ n->w = 96*4; n->h = 96*4; }else
		{ n->w = 96; n->h = 96; }
		n->draw = ((n->shown==TRUE) && (cnt2>0))?'1':'0';
	}

	// draw connecting lines
	gdk_gc_set_foreground(gc, &black);
	for (ln=data->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->draw=='1')
		{
			if ((n->subtype==network_p2p) && (n->interfaces.size()==2))
			{
				draw_ptop(canvas, scale, n);
			}else
			if (n->subtype==network_ring)
			{
				draw_fddi(canvas, scale, n);
			}else
			if (n->subtype==network_hbus)
			{
				draw_hbus(canvas, scale, n, width, height, xp, yp);
			}else
			if (n->subtype==network_vbus)
			{
				draw_vbus(canvas, scale, n, width, height, xp, yp);
			}else
			if (n->subtype==network_cloud)
			{
				for (ln2=n->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
				{
					in = (Interface*) ln2->item;
					r = in->device;
					if ((r!=NULL) && (r->shown==TRUE))
					{
						if (showcolourlines==TRUE)
							interface_color(gc, in->type);
						else
							gdk_gc_set_foreground(gc, &black);
						gdk_draw_line(canvas, gc,
							gint(n->rx), gint(n->ry), gint(r->rx), gint(r->ry));
					}
				}
				if (n->group!=network_group_bus) // draw lines for snake bus elsewhere
				{
					for (ln3=n->interfaces.get_head(); ln3!=NULL; ln3=ln3->next)
					{
						in = (Interface*) ln3->item;
						for (ln2=in->machines.get_head(); ln2!=NULL; ln2=ln2->next)
						{
							m = (Machine*) ln2->item;
							if (m->shown==TRUE)
							{
								gdk_gc_set_foreground(gc, &black);
								gdk_draw_line(canvas, gc,
									gint(n->rx), gint(n->ry), gint(m->rx), gint(m->ry));
							}
						}
					}
				}
			}
/*
			// now draw lines to machines
			for (ln2=n->machines.get_head(); ln2!=NULL; ln2=ln2->next)
			{
				m = (Machine*) ln2->item;
				if (m->shown==TRUE)
				{
					gdk_gc_set_foreground(gc, &black);
					gdk_draw_line(canvas, gc,
						gint(n->rx), gint(n->ry), gint(m->rx), gint(m->ry));
				}
			}
*/
		}
	}

	// draw network clouds
	for (ln=data->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->draw=='1')
		{
			if (n->subtype==network_cloud)
			{
				// draw little circle
				gdk_gc_set_foreground(gc, &network_color);
				if (n->selection==TRUE)
					gdk_gc_set_foreground(gc, &network_selected_color);
				if (n->numMachines()==0)
				{
					gdk_gc_set_foreground(gc, &network_empty_color);
					if (n->selection==TRUE)
						gdk_gc_set_foreground(gc, &network_selected_color);
				}
				gdk_draw_arc(canvas, gc, TRUE,
					int(n->rx-(n->w/(2*scale))), int(n->ry-(n->h/(2*scale))),
					int(n->w/scale), int(n->h/scale), 0, 360*64);
				lcloud.draw(canvas,
					int(n->rx-(n->w/(2*scale))),
					int(n->ry-(n->h/(2*scale))));
				// draw black edge
//				gdk_gc_set_foreground(gc3, &black);
//				gdk_draw_arc(canvas, gc3, FALSE,
//					int(n->rx-(n->w/(2*scale))), int(n->ry-(n->h/(2*scale))),
//					int(n->w/scale), int(n->h/scale), 0, 360*64);
			}
		}
	}

	// draw machines
	for (ln=data->machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		if ((m->shown==TRUE) && (m->interface!=NULL) &&
				(m->interface->network!=NULL) && (m->interface->network->shown==TRUE))
		{
			gdk_gc_set_foreground(gc, &machine_color);
			if (m->selection==TRUE)
				gdk_gc_set_foreground(gc, &machine_selected_color);
			gdk_draw_rectangle(canvas, gc, TRUE,
				int(m->rx-(m->w/(2*scale))), int(m->ry-(m->h/(2*scale))),
				int(m->w/scale), int(m->h/scale));
			if (m->subtype==machine_printer)
			{
				lprinter.draw(canvas,
					int(m->rx-(m->w/(2*scale))),
					int(m->ry-(m->h/(2*scale))));
			}else
			if (m->subtype==machine_server)
			{
				lserver.draw(canvas,
					int(m->rx-(m->w/(2*scale))),
					int(m->ry-(m->h/(2*scale))));
			}else
			{
				lmachine.draw(canvas,
					int(m->rx-(m->w/(2*scale))),
					int(m->ry-(m->h/(2*scale))));
			}
			if ((pp==0) && (showlabels==TRUE) && (fntsz>=6))
			{
				i = strlen(m->name);
				if (i>0)
				{
					strcpy(buf, m->name);
					for (j=0; j<i; j++) if (buf[j]=='.') buf[j]='\0';
				}else
				{
					strcpy(buf, m->ip.getstring());
				}
				ww = gdk_text_width(fnt2, buf, strlen(buf));
				labelx = (int) (m->rx-(ww/2));
				labely = (int) ((m->ry+((m->h/2)-4)/scale)+fnt->ascent);
				gdk_gc_set_foreground(gc, &white);
				gdk_draw_text(canvas, fnt2, gc, labelx-1, labely-1,
					buf, strlen(buf));
				gdk_draw_text(canvas, fnt2, gc, labelx+1, labely-1,
					buf, strlen(buf));
				gdk_draw_text(canvas, fnt2, gc, labelx+1, labely+1,
					buf, strlen(buf));
				gdk_draw_text(canvas, fnt2, gc, labelx-1, labely+1,
					buf, strlen(buf));
				gdk_gc_set_foreground(gc, &black);
				gdk_draw_text(canvas, fnt2, gc, labelx, labely,
					buf, strlen(buf));
			}
		}
	}

	// draw routers & switches
	for (ln=data->devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		if (r->shown==TRUE)
		{
			// draw main circle
			if (r->status=='1')
			{
				gdk_gc_set_foreground(gc, &device_up_color);
				if (r->selection==TRUE)
					gdk_gc_set_foreground(gc, &device_up_selected_color);
			}else
			{
				gdk_gc_set_foreground(gc, &device_down_color);
				if (r->selection==TRUE)
					gdk_gc_set_foreground(gc, &device_down_selected_color);
			}
			gdk_draw_arc(canvas, gc, TRUE,
				int(r->rx-(r->w/(2*scale))), int(r->ry-(r->h/(2*scale))),
				int(r->w/scale), int(r->h/scale), 0, 360*64);

			// draw graphics
			if (r->subtype==device_switch)
			{
				// scales size of image not size of device
				lswitch.draw(canvas,
					int(r->rx-(144/(2*scale))),
					int(r->ry-(144/(scale))));				
			}else
			if (r->subtype==device_merged)
			{
				lbrouter.draw(canvas,
					int(r->rx-(144/(2*scale))),
					int(r->ry-(144/(scale))));				
			}else
			if (r->subtype==device_router)
			{
				lrouter.draw(canvas,
					int(r->rx-(144/(2*scale))),
					int(r->ry-(144/(scale))));				
			}else
			{
				lunknown.draw(canvas,
					int(r->rx-(144/(2*scale))),
					int(r->ry-(144/(scale))));				
			}

			// draw black edge
//			gdk_gc_set_foreground(gc3, &black);
//			gdk_draw_arc(canvas, gc3, FALSE,
//				int(r->rx-(r->w/(2*scale))), int(r->ry-(r->h/(2*scale))),
//				int(r->w/scale), int(r->h/scale), 0, 360*64);

			if ((pp==0) && (showlabels==TRUE) && (fntsz>=2))
			{
				strcpy(buf, r->name);
				j=strlen(buf); for (i=0; i<j; i++) if (buf[i]=='.') buf[i]='\0';
				ww = gdk_text_width(fnt, buf, strlen(buf));
				labelx = (int) (r->rx-(ww/2));
				labely = (int) ((r->ry+((r->h/2)+4)/(2*scale)));
//				labely = (int) ((r->ry+((r->h/2))/(2*scale))+fnt->ascent);
				gdk_gc_set_foreground(gc, &white);
				gdk_draw_text(canvas, fnt, gc, labelx-1, labely-1,
					buf, strlen(buf));
				gdk_draw_text(canvas, fnt, gc, labelx+1, labely-1,
					buf, strlen(buf));
				gdk_draw_text(canvas, fnt, gc, labelx+1, labely+1,
					buf, strlen(buf));
				gdk_draw_text(canvas, fnt, gc, labelx-1, labely+1,
					buf, strlen(buf));
				gdk_gc_set_foreground(gc, &black);
				gdk_draw_text(canvas, fnt, gc, labelx, labely,
					buf, strlen(buf));
			}
		}
	}
	gdk_font_unref(fnt);
	gdk_font_unref(fnt2);
}

void Diagram::draw_ptop(GdkPixmap *canvas, float scale, Network *n)
{
	Device *d1, *d2;
	Interface *in;
	int xdist, ydist;
	double zdist, hdist, vdist;

	if (n->interfaces.size()!=2) return;

	in = (Interface*) n->interfaces.get_item(0); d1 = in->device;
	in = (Interface*) n->interfaces.get_item(1); d2 = in->device;
	if ((d1->shown==FALSE) || (d2->shown==FALSE)) return;

	xdist = gint(d1->rx - d2->rx);
	ydist = gint(d1->ry - d2->ry);
	zdist = sqrt((xdist*xdist)+(ydist*ydist));
	hdist = (xdist*(d1->w/(8*scale)))/zdist;
	vdist = (ydist*(d1->w/(8*scale)))/zdist;

	if (showcolourlines==TRUE)
		interface_color(gc, in->type);
	else
		gdk_gc_set_foreground(gc, &black);

/*
	gdk_draw_line(canvas, gc,
		gint(d1->rx), gint(d1->ry),
		gint(d1->rx - (xdist/2) + vdist), gint(d1->ry - (ydist/2) - hdist));
	gdk_draw_line(canvas, gc,
		gint(d1->rx - (xdist/2) + vdist), gint(d1->ry - (ydist/2) - hdist),
		gint(d1->rx - (xdist/2) - vdist), gint(d1->ry - (ydist/2) + hdist));
	gdk_draw_line(canvas, gc,
		gint(d1->rx - (xdist/2) - vdist), gint(d1->ry - (ydist/2) + hdist),
		gint(d2->rx), gint(d2->ry));
*/



	points[0].x = gint(d1->rx);
	points[0].y = gint(d1->ry);
	points[1].x = gint(d1->rx - (xdist/2) + vdist);
	points[1].y = gint(d1->ry - (ydist/2) - hdist);
	points[2].x = gint(d1->rx - (xdist/2) - vdist);
	points[2].y = gint(d1->ry - (ydist/2) + hdist);
	points[3].x = gint(d2->rx);
	points[3].y = gint(d2->ry);
	gdk_draw_lines(canvas, gc, points, 4);
}

void Diagram::draw_fddi(GdkPixmap *canvas, float scale, Network *n)
{
	Device *r;
	Machine *m;
	Interface *in;
	int xdist, ydist;
	double zdist, hdist, vdist;
	Link *ln, *ln2;


	if (showcolourlines==TRUE)
		interface_color(gc, in->type);
	else
		gdk_gc_set_foreground(gc, &black);

	for (ln=n->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		r = in->device;

		if (r->shown==TRUE)
		{
			xdist = gint(r->rx - n->rx);
			ydist = gint(r->ry - n->ry);
			zdist = sqrt((xdist*xdist)+(ydist*ydist));
			hdist = (xdist*(r->w/(scale)))/zdist;
			vdist = (ydist*(r->w/(scale)))/zdist;

			gdk_draw_line(canvas, gc,
				gint(r->rx), gint(r->ry),
				gint(n->rx + hdist), gint(n->ry + vdist));
		}

		for (ln2=in->machines.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			m = (Machine*) ln2->item;

			if (m->shown==TRUE)
			{
				xdist = gint(m->rx - n->rx);
				ydist = gint(m->ry - n->ry);
				zdist = sqrt((xdist*xdist)+(ydist*ydist));
				hdist = (xdist*((m->w*2)/(scale)))/zdist;
				vdist = (ydist*((m->w*2)/(scale)))/zdist;

				gdk_draw_line(canvas, gc,
					gint(m->rx), gint(m->ry),
					gint(n->rx + hdist), gint(n->ry + vdist));
			}
		}
	}

	gdk_draw_arc(canvas, gc, FALSE,
		int(n->rx-(n->w/(2*scale))), int(n->ry-(n->h/(2*scale))),
		int((n->w)/scale), int((n->h)/scale), 0, 360*64);
}

void Diagram::draw_hbus(GdkPixmap *canvas, float scale, Network *n, long width, long height, long xp, long yp)
{
	Device *r;
	Machine *m;
	Interface *in;
	int count, xmin, xmax, ymin, ymax;
	Link *ln, *ln2;

	if (n->interfaces.size()<1) return;

	count=0;
	for (ln=n->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		r = in->device;
		if (r->shown==TRUE)
		{
			if (count==0)
			{
				xmin = xmax = (int) r->x;
			}else
			{
				if (xmax<r->x) xmax=(int)r->x;
				if (xmin>r->x) xmin=(int)r->x;
			}
			count++;
		}
		for (ln2=in->machines.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			m = (Machine*) ln2->item;

			if (m->shown==TRUE)
			{
				if (count==0)
				{
					xmin = xmax = (int) m->x;
				}else
				{
					if (xmax<m->x) xmax=(int)m->x;
					if (xmin>m->x) xmin=(int)m->x;
				}
				count++;
			}
		}
	}

	if (count==0) return;

	// at this point r still points to a device
	// set size & coordinates of bus
	n->w = xmax - xmin + r->w;
	n->h = (r->h/4);
	n->x = xmin + (xmax - xmin)/2;		// xmin - (r->w/2);
	// keep n->y

	// convert x -> rx & y -> ry
	n->rx = (n->x/scale)+(width/2)-(xp/scale);
	n->ry = (n->y/scale)+(height/2)-(yp/scale);

	if (showcolourlines==TRUE)
		interface_color(gc, in->type);
	else
		gdk_gc_set_foreground(gc, &black);

	// now draw the bus
	gdk_draw_line(canvas, gc,
		gint(n->rx-(n->w/(2*scale))), gint(n->ry),
		gint(n->rx+(n->w/(2*scale))), gint(n->ry));
	for (ln=n->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		r = in->device;

		if (r->shown==TRUE)
		{
			gdk_draw_line(canvas, gc,
				gint(r->rx), gint(r->ry),
				gint(r->rx), gint(n->ry));
		}
	}
	for (ln2=n->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
	{
		in = (Interface*) ln2->item;
		for (ln=in->machines.get_head(); ln!=NULL; ln=ln->next)
		{
			m = (Machine*) ln->item;
			if (m->shown==TRUE)
			{
				gdk_draw_line(canvas, gc,
					gint(m->rx), gint(m->ry),
					gint(m->rx), gint(n->ry));
			}
		}
	}
}

void Diagram::draw_vbus(GdkPixmap *canvas, float scale, Network *n, long width, long height, long xp, long yp)
{
	Device *r;
	Machine *m;
	Interface *in;
	int count, xmin, xmax, ymin, ymax;
	Link *ln, *ln2;

	if (n->interfaces.size()<1) return;

	// calculate the position of the ends of the line representing the bus
	count=0;
	for (ln=n->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		r = in->device;
		if (r->shown==TRUE)
		{
			if (count==0)
			{
				ymin = ymax = (int) r->y;
			}else
			{
				if (ymax<r->y) ymax=(int)r->y;
				if (ymin>r->y) ymin=(int)r->y;
			}
			count++;
		}
		for (ln2=in->machines.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			m = (Machine*) ln2->item;

			if (m->shown==TRUE)
			{
				if (count==0)
				{
					ymin = ymax = (int) m->y;
				}else
				{
					if (ymax<m->y) ymax=(int)m->y;
					if (ymin>m->y) ymin=(int)m->y;
				}
				count++;
			}
		}
	}

	if (count==0) return;

	// at this point r still points to a device
	// set size & coordinates of bus
	n->w = (r->w/4);
	n->h = ymax - ymin + r->h;
	// keep n->x
	n->y = ymin + (ymax - ymin)/2;		// ymin - (r->h/2);

	// convert x -> rx & y -> ry
	n->rx = (n->x/scale)+(width/2)-(xp/scale);
	n->ry = (n->y/scale)+(height/2)-(yp/scale);

	if (showcolourlines==TRUE)
		interface_color(gc, in->type);
	else
		gdk_gc_set_foreground(gc, &black);

	// now draw the bus

	gdk_draw_line(canvas, gc,
		gint(n->rx), gint(n->ry-(n->h/(2*scale))),
		gint(n->rx), gint(n->ry+(n->h/(2*scale))));
	for (ln=n->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		r = in->device;

		if (r->shown==TRUE)
		{
			gdk_draw_line(canvas, gc,
				gint(r->rx), gint(r->ry),
				gint(n->rx), gint(r->ry));
		}
	}
	for (ln2=n->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
	{
		in = (Interface*) ln2->item;
		for (ln=in->machines.get_head(); ln!=NULL; ln=ln->next)
		{
			m = (Machine*) ln->item;
			if (m->shown==TRUE)
			{
				gdk_draw_line(canvas, gc,
					gint(m->rx), gint(m->ry),
					gint(n->rx), gint(m->ry));
			}
		}
	}
}

