/* message.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <stdio.h>
#include "message.h"
#include "gui.h"
#include "licon.xpm"

static gint delete_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{ ((Message*)data)->hide(); return TRUE; }

Message::Message()
{
	window = gtk_dialog_new();
  gtk_window_set_modal (GTK_WINDOW (window), TRUE);
  gtk_window_set_policy (GTK_WINDOW (window), FALSE, FALSE, TRUE);
	gtk_container_border_width(GTK_CONTAINER(window), 5);
	gtk_widget_realize(window);
	window_icon(window, licon_xpm);
	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
		GTK_SIGNAL_FUNC(delete_cb), this);
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
		GTK_SIGNAL_FUNC(delete_cb), this);

  label = gtk_label_new ("Message");
  gtk_widget_show (label);
	gtk_misc_set_padding(GTK_MISC(label), 10, 10);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window)->vbox), label, TRUE, TRUE, 0);

  button_yes = gtk_button_new_with_label ("Yes");
  gtk_widget_show (button_yes);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window)->action_area), button_yes, FALSE, FALSE, 0);
  gtk_widget_set_usize (button_yes, 60, -2);

  button_no = gtk_button_new_with_label ("No");
	gtk_widget_show (button_no);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window)->action_area), button_no, FALSE, FALSE, 0);
  gtk_widget_set_usize (button_no, 60, -2);

  button_cancel = gtk_button_new_with_label ("Cancel");
  gtk_widget_show (button_cancel);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window)->action_area), button_cancel, FALSE, FALSE, 0);
  gtk_widget_set_usize (button_cancel, 60, -2);
}

Message::~Message()
{
}

void Message::show()
{
	gtk_widget_show(window);
	gdk_window_raise(window->window);
}

void Message::hide()
{
	gtk_widget_hide(window);
}

void Message::setTitle(char *str)
{
  gtk_window_set_title(GTK_WINDOW (window), str);
	gdk_window_set_icon_name(window->window, str);
}

void Message::setLabel(char *str)
{
	gtk_label_set_text(GTK_LABEL(label), str);
}

void Message::setButtons(int num)
{
	switch (num)
	{
		case 1:
			gtk_label_set_text(GTK_LABEL(GTK_BIN(button_yes)->child), "Yes");
			gtk_label_set_text(GTK_LABEL(GTK_BIN(button_no)->child), "No");
			gtk_label_set_text(GTK_LABEL(GTK_BIN(button_cancel)->child), "Cancel");
			gtk_widget_show(button_yes);
			gtk_widget_show(button_no);
			gtk_widget_show(button_cancel);
			break;
		case 2:
			gtk_label_set_text(GTK_LABEL(GTK_BIN(button_yes)->child), "Yes");
			gtk_label_set_text(GTK_LABEL(GTK_BIN(button_no)->child), "No");
			gtk_widget_show(button_yes);
			gtk_widget_show(button_no);
			gtk_widget_hide(button_cancel);
			break;
		case 3:
			gtk_label_set_text(GTK_LABEL(GTK_BIN(button_yes)->child), "OK");
			gtk_widget_show(button_yes);
			gtk_widget_hide(button_no);
			gtk_widget_hide(button_cancel);
			break;
		default: break;
	}
}

