/* parser.cc
 * Copyright (C) 2001-2003 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <arpa/inet.h>
#include <string.h>
#include <netdb.h>
#include <stdio.h>
#include "parser.h"
#include "data.h"
#include "misc.h"

Parser::Parser()
{
	empty();
}

Parser::~Parser()
{
	empty();
}

void Parser::empty()
{
	Link *ln;
	SubnetExtra *s;
	Port *p;

	if (d!=NULL) delete d; d = NULL;
	for (ln=subnets.get_head(); ln!=NULL; ln=ln->next)
	{ s = (SubnetExtra*) ln->item; if (s!=NULL) delete s; }
	subnets.empty();
	for (ln=ports.get_head(); ln!=NULL; ln=ln->next)
	{ p = (Port*) ln->item; if (p!=NULL) delete p; }
	ports.empty();
}

void Parser::load_device(char *fname, char *pname)
{
	FILE *f;
	char buf[1024], var[1024], val[1024], dscr[1024];
	int s;
	IPAddress tip;
	struct in_addr addr;
	struct hostent *h;

	empty();

	f = fopen(fname, "r");
	if (!f) return;

	d = new Device();
	d->pingname.setstring(pname);
	strcpy(d->name, pname);
	if (inet_aton(pname, &addr) != 0)
	{
		h = gethostbyaddr((const char *) &addr.s_addr, sizeof(addr), AF_INET);
		if (h) strcpy(d->name, h->h_name);
	}
	fscanf(f, "%[^\n]\n", buf);
	d->last_seen.setstring(buf);
	fscanf(f, "%[^\n]\n", d->community);

	while(fscanf(f, "%[^\n]\n", buf)==1)
	{
		if (sscanf(buf, "%[^ ] %[^\n]", var, val)==2)
		{
			if (strcmp("iso.3.6.1.2.1.1.1.0", var)==0)
			{
				val[strlen(val)-1]='\0';
				strcpy(dscr, val+1);
			}else
			if (strcmp("iso.3.6.1.2.1.1.7.0", var)==0)
			{
				if (sscanf(val, "%d", &s)==1)
				{
					getDescr(dscr, s);
				}
			}else
			if (strcmp("iso.3.6.1.2.1.1.5.0", var)==0)
			{
				// system name overrides DNS name because
				// of the way auto discovery works
				val[strlen(val)-1]='\0';
				strcpy(d->name, val+1);
			}else
			if (strcmp("iso.3.6.1.2.1.1.4.0", var)==0)
			{
				val[strlen(val)-1]='\0';
				strcpy(d->contact, val+1);
			}else
			if (strcmp("iso.3.6.1.2.1.1.6.0", var)==0)
			{
				val[strlen(val)-1]='\0';
				strcpy(d->location, val+1);
			}else
			if (starts_with("iso.3.6.1.2.1.2.2.1", var))
			{
				add_to_interface(var, val);
			}else
			if (starts_with("iso.3.6.1.2.1.31.1.1.1", var))
			{
				add_to_interface_extended(var, val);
			}else
			if (starts_with("iso.3.6.1.2.1.4.20.1", var))
			{
				add_to_ips(var, val);
			}else
			if (starts_with("iso.3.6.1.2.1.4.21.1", var))
			{
				add_to_route(var, val);
			}else
			if (starts_with("iso.3.6.1.2.1.4.22.1.2", var))
			{
				add_to_address(var, val);
			}else
			if (starts_with("iso.3.6.1.2.1.17.1.4.1.2", var))
			{
				add_to_bridge_map(var, val);
			}else
			if (starts_with("iso.3.6.1.2.1.17", var))
			{
				add_to_bridge(var, val);
			}
		}
	}
//	print();
}

void Parser::getDescr(char *str, int val)
{
	FILE *f;
	char comm[64], tmpbuf[64], lst[256], buff[256];
	gchar *fname;
	char ch;
	int mask, serv, len, i, pos, status;

	fname = user_file("device-types");
	if (fname!=NULL)
	{
		f = fopen(fname, "r");
		if (f)
		{
			while (fgets(buff, 1023, f))
			{
				if ((buff[0]!='\n') && (buff[0]!='#'))
				{
					if ((sscanf(buff, "%c \"%[^\"]\" %d %d %[^\n]\n",
									&ch, comm, &mask, &serv, lst)==5) ||
							(sscanf(buff, "%c \"%[^\"]\" %d %d\n",
									&ch, comm, &mask, &serv)==4))
					{
						status = 1;

//printf("val=%d, mask=%d val&mask=%d serv=%d\n", val, mask, val&mask, serv);
//printf("str=%s\nlst=%s\n", str, lst);
						if ((val&mask)!=serv) status = 0;
						
						len = strlen(lst);
						pos = 0;
						for (i=0; i<len; i++)
						{
							if (lst[i]!=' ')
							{
								tmpbuf[pos] = lst[i];
								pos++;
							}else
							{
								tmpbuf[pos] = '\0';
								if (!lower_contains(str, tmpbuf)) status = 0;
								pos = 0;
							}
						}
						if (pos>0)
						{
							tmpbuf[pos] = '\0';
							if (!lower_contains(str, tmpbuf)) status = 0;
						}

						if (status==1)
						{
//printf("MATCH %s, %c\n", comm, ch);
							strcpy(d->descr, comm);
							d->maintype = visual_device;
							// FIXME handle all other data types. Really we need to rewrite
							// this whole device detection code for more detailed detection
							if (ch=='r') d->subtype = device_router; else
							if (ch=='s') d->subtype = device_switch; else
								d->subtype = device_unknown;
						}
					}
				}
			}
		}
	}
}

void Parser::add_to_interface(char *var, char *val)
{
	char vartype[256], junk[256];
	Interface *in, *tin;
	Link *ln;
	unsigned long s;
	int j, ifnum;

	if (sscanf(var, "iso.3.6.1.2.1.2.2.1.%[^.].%d", vartype, &ifnum)!=2)
		return;

	tin = NULL;
	for (ln=d->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		if (in->index == ifnum) tin = in;
	}

	if (tin == NULL)
	{
		tin = new Interface();
		tin->index = ifnum;
		tin->device = d;
		d->interfaces.tag(tin);
	}

	if ((strcmp("5", vartype)==0) && (sscanf(val, "%lu", &s)==1))
		tin->speed = s;
	else
	if ((strcmp("2", vartype)==0) && (sscanf(val, "\"%[^\"]\"", junk)==1))
		strcpy(tin->descr, junk);
	else
	if ((strcmp("3", vartype)==0) && (sscanf(val, "%d", &j)==1))
		tin->type = j;
	else
	if ((strcmp("8", vartype)==0) && (sscanf(val, "%[^\n]", junk)==1))
		tin->status = junk[0];
	else
	if (strcmp("6", vartype)==0)
		tin->mac.setstringsnmp(val);
}

void Parser::add_to_interface_extended(char *var, char *val)
{
	char vartype[256], junk[256];
	Interface *in, *tin;
	Link *ln;
	int ifnum;

	if (sscanf(var, "iso.3.6.1.2.1.31.1.1.1.%[^.].%d", vartype, &ifnum)!=2)
		return;

	tin = NULL;
	for (ln=d->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		if (in->index == ifnum) tin = in;
	}

	if (tin == NULL)
	{
		tin = new Interface();
		tin->index = ifnum;
		tin->device = d;
		d->interfaces.tag(tin);
	}

	if (strcmp("1", vartype)==0)
	{
		if ((sscanf(val, "\"%[^\"]\"", junk)==1) && (strlen(junk)>0))
			strcpy(tin->descr, junk);
	}else
	if (strcmp("18", vartype)==0)
	{
		if ((sscanf(val, "\"%[^\"]\"", junk)==1) && (strlen(junk)>0))
			strcpy(tin->comment, junk);
	}
}

void Parser::add_to_ips(char *var, char *val)
{
	char vartype[256], idxval[256];
	Interface *in, *tin;
	Link *ln;
	IPAddress *tip;
	int ifnum;

	if (sscanf(val, "%d", &ifnum)!=1) return;
	if (sscanf(var, "iso.3.6.1.2.1.4.20.1.%[^.].%[^\n]", vartype, idxval)!=2)
		return;

	tin = NULL;
	for (ln=d->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		if (in->index == ifnum) tin = in;
	}

	if (tin != NULL)
	{
		if (strcmp("2", vartype)==0)
		{
			tip = new IPAddress();
			tip->setstring(idxval);
			tin->ip_addresses.tag(tip);
		}
	}
}

void Parser::add_to_route(char *var, char *val)
{
	char vartype[256], idxval[256];
	SubnetExtra *sn, *tsn;
	IPAddress tip, tmsk;
	Link *ln;
	int j;

	if (sscanf(var, "iso.3.6.1.2.1.4.21.1.%[^.].%[^\n]", vartype, idxval)!=2)
		return;

	tip.setstring(idxval);

	tsn = NULL;
	for (ln=subnets.get_head(); ln!=NULL; ln=ln->next)
	{
		sn = (SubnetExtra*) ln->item;
		if (sn->ip.compare(tip)==0) tsn = sn;
	}

	if (tsn == NULL)
	{
		tsn = new SubnetExtra();
		tsn->ip.set(tip);
		tsn->mask = 0;
		tsn->next.empty();
		tsn->index = 0;
		tsn->type = 0;
		subnets.tag(tsn);
	}

	if ((strcmp("2", vartype)==0) && (sscanf(val, "%d", &j)==1))
	{
		tsn->index = j;
	}else
	if (strcmp("11", vartype)==0)
	{
		tmsk.setstring(val);
		tsn->mask = tmsk.mask_value();
	}else
	if (strcmp("7", vartype)==0)
	{
		tsn->next.setstring(val);
	}else
	if ((strcmp("8", vartype)==0) && (sscanf(val, "%d", &j)==1))
	{
		tsn->type = j;
	}
}

void Parser::add_to_address(char *var, char *val)
{
	char vartype[256], junk[256];
	MACAddress tmac;
	Machine *m;
	IPAddress tip;
	Interface *in;
	Link *ln;
	int ifnum;

	if (sscanf(var, "iso.3.6.1.2.1.4.22.1.2.%[^.].%[^\n]", vartype, junk)!=2)
		return;
	if (sscanf(vartype, "%d", &ifnum)!=1) return;

	tmac.setstringsnmp(val);
	tip.setstring(junk);
	if (tip.in_range("127.0.0.0", "127.0.0.255"))
	{
//		mm = new MACAddress(tmac);
//		d->loopbacks.tag(mm);
	}else
	{
		m = new Machine();
		m->mac = tmac;
		m->ip = tip;
		for (ln=d->interfaces.get_head(); ln!=NULL; ln=ln->next)
		{
			in = (Interface*) ln->item;
			if (in->index == ifnum) in->machines.tag(m);
			m->interface = in;
		}
	}
}

void Parser::add_to_bridge_map(char *var, char *val)
{
	int j, ifnum;
	Port *p, *tmpp;
	Link *ln;

	if (sscanf(var, "iso.3.6.1.2.1.17.1.4.1.2.%d", &ifnum)!=1) return;
	sscanf(val, "%d", &j);

	tmpp = NULL;
	for (ln=ports.get_head(); ln!=NULL; ln=ln->next)
	{
		p = (Port*) ln->item;
		if (p->port == ifnum) tmpp = p;
	}

	if (tmpp==NULL)
	{
		p = new Port();
		p->port = ifnum;
		ports.tag(p);
	}

	p->index = j;
}

void Parser::add_to_bridge(char *var, char *val)
{
	char newmac[64];
	MACAddress tmpmac;
	Interface *in;
	Port *pt;
	Machine *m, *tmpm;
	int j, k;
	Link *ln, *ln2;

	// a bridge variable is split into 2
	if (sscanf(var, "iso.3.6.1.2.1.17.4.3.1.2.%[^\n]", newmac)!=1) return;
	tmpmac.setstringsnmp2(newmac);
	sscanf(val, "%d", &j);

	// if MAC address is 000000000000 then reject
	if (tmpmac.zero()) return;

	// scan all machines on device
	tmpm = NULL;
	for (ln=d->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		for (ln2=in->machines.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			m = (Machine*) ln2->item;
			if (m->mac.compare(tmpmac)==1) tmpm = m;
		}
	}

	if (tmpm == NULL)
	{
		m = new Machine();
		m->mac = tmpmac;
		k=-1;
		for (ln=ports.get_head(); ln!=NULL; ln=ln->next)
		{
			pt = (Port*) ln->item;
			if (pt->port == j)
				k = pt->index;
		}
		if (k!=-1)
		{
			for (ln=d->interfaces.get_head(); ln!=NULL; ln=ln->next)
			{
				in = (Interface*) ln->item;
				if (in->index == k)
				{
					in->machines.tag(m);
					m->interface = in;
				}
			}			
		}
		if (m->interface==NULL)
		{
//			printf("Failed to attach machine ... %s\n", m->mac.getstring());
			// these are either references to the devices own interfaces OR
			// entries on interface NULL which we just throw away
			delete m;
		}
	}
}

void Parser::print()
{
	printf("Parser Output\n=============\n");
	printf("Name        : %s\n", d->name);
	printf("IP Address  : %s\n", d->pingname.getstring());
	printf("Type        : %c %c\n", d->maintype, d->subtype);
	printf("Last Seen   : %s\n", d->last_seen.date_to_str(1));
	printf("Location    : %s\n", d->location);
	printf("Community   : %s\n", d->community);
	printf("Description : %s\n", d->descr);
	printf("Comment     : %s\n", d->comment);
	printf("Contact     : %s\n", d->contact);
}

