/* tasklist.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <glade/glade-xml.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include "session.h"
#include "tasklist.h"
#include "main.h"
#include "misc.h"
#include "gui.h"
#include "scheduler.h"
#include "task.h"
#include "licon.xpm"

static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{ ((TaskList*)data)->hide(); return (TRUE); }
static void close_cb(GtkWidget *w, gpointer data)
{ ((TaskList*)data)->hide(); }

TaskList::TaskList(Main *m)
{
	GladeXML *gxml;

	main = m;

	gchar *fname = user_file("nomad.glade");
	gxml = glade_xml_new (fname, "tasklist");
	if (!gxml)
	{
		g_warning("Failed to load file nomad.glade");
		return;
	}

	window = glade_xml_get_widget (gxml, "tasklist");
	button7 = glade_xml_get_widget (gxml, "button7");
	clist5 = glade_xml_get_widget (gxml, "clist5");
	gtk_object_unref (GTK_OBJECT (gxml));
	g_free(fname);

	gtk_widget_set_usize (window, 420, 280);
	gtk_widget_realize (window);
	window_icon(window, licon_xpm);
	gdk_window_set_icon_name(window->window, "Task List");
	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
		GTK_SIGNAL_FUNC(delete_event), this);
	gtk_signal_connect (GTK_OBJECT (button7), "pressed",
		GTK_SIGNAL_FUNC (close_cb), this);
}

TaskList::~TaskList()
{
}

void TaskList::show()
{
	gtk_widget_show(window);
	gdk_window_raise(window->window);
}

void TaskList::hide()
{
	gtk_widget_hide(window);
}

void TaskList::update()
{
	Link *ln;
	Task *t;
	gchar *data[2];
	char buf[256], str[256];
	int i, j, lvl, row = 0;
	gtk_clist_freeze(GTK_CLIST(clist5));
	gtk_clist_clear(GTK_CLIST(clist5));
	for (lvl=0; lvl<=6; lvl++)
	{
		for (ln=main->scheduler->tasks.get_head(); ln!=NULL; ln=ln->next)
		{
			t = (Task*) ln->item;
			if (t->priority==lvl)
			{
				if (t->type == 2)
					sprintf(buf, "Pinging Network Devices"); else
				if (t->type == 1)
				{
					strcpy(str, t->cmnd);
					j = strlen(str); for (i=0; i<j; i++) if (str[i]==' ') str[i]='\0';
					sprintf(buf, "Grabbing Topology From Device (%s)", str);
				}else
					sprintf(buf, "");
				data[0] = strdup(buf);
				data[1] = strdup((t->pid==0)?"Queued":"Active");
				gtk_clist_append(GTK_CLIST(clist5), data);
				delete data[0]; delete data[1];
				row++;
			}
		}
	}
	gtk_clist_thaw(GTK_CLIST(clist5));

	for (i=0; i<row; i++)
	{
		gtk_clist_set_background(GTK_CLIST(clist5), i, ((i%2)?(&main->gui->list_background_color_1):(&main->gui->list_background_color_2)));
	}
}

