/* viewer.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gnome.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include "main.h"
#include "gui.h"
#include "data.h"
#include "session.h"
#include "scheduler.h"
#include "topology.h"
#include "viewer.h"
#include "monitor.h"
#include "preferences.h"
#include "viewsidebar.h"
#include "diagram.h"
#include "network.h"
#include "interface.h"
#include "machine.h"
#include "device.h"
#include "networkdetails.h"
#include "machinedetails.h"
#include "devicedetails.h"
#include "devicesidebar.h"
#include "searchsidebar.h"
#include "list.h"
#include "sfitall.xpm"
#include "sfitsel.xpm"
#include "sshowall.xpm"

/***************************************************************************
 Callbacks
 ***************************************************************************/

static void details_cb(GtkWidget *w, gpointer data)
{ ((Viewer*)data)->details_action(); }

static void delete_cb(GtkWidget *w, gpointer data)
{
	Viewer *v = (Viewer*)data;
	if ((v->cur!=NULL) && (v->cur->selection==FALSE) &&
			(v->cur->maintype==visual_device))
		v->main->topology->deviceDelete((Device*) v->cur);
	else
		v->main->topology->deviceDeleteSelected();
}

static void quicksearch_cb(GtkWidget *w, gpointer data)
{
	Viewer *v = (Viewer*)data;
	gchar *txt = gtk_entry_get_text(GTK_ENTRY(w));
	if (strcmp(txt, "")==0)
	{
		v->main->session->data->show_all();
		v->main->gui->update_views();
	}else
		v->main->gui->monitor->searchsidebar->quicksearch((char*)txt);
	v->draw();
}
static void show_leaf_networks_cb(GtkWidget *w, gpointer data)
{ ((Viewer*)data)->leaf_networks(TRUE); }
static void hide_leaf_networks_cb(GtkWidget *w, gpointer data)
{ ((Viewer*)data)->leaf_networks(FALSE); }
static void show_leaf_machines_cb(GtkWidget *w, gpointer data)
{ ((Viewer*)data)->leaf_machines(TRUE); }
static void hide_leaf_machines_cb(GtkWidget *w, gpointer data)
{ ((Viewer*)data)->leaf_machines(FALSE); }

static void network_graph_cb(GtkWidget *w, gpointer data)
{ ((Viewer*)data)->main->gui->graph_window(); }

static void device_graph_cb(GtkWidget *w, gpointer data)
{ ((Viewer*)data)->main->gui->graph_window(); }
static void merge_cb(GtkWidget *w, gpointer data)
{ ((Viewer*)data)->main->topology->deviceMerge(); }
static void split_cb(GtkWidget *w, gpointer data)
{
	Device *d;
	Viewer *v = (Viewer*)data;
	if ((v->cur!=NULL) && (v->cur->maintype==visual_device))
	{
		d = (Device*) v->cur;
		if (d->subtype==device_merged)
			v->main->topology->deviceSplit(d);
	}
}
static void hide_cb(GtkWidget *w, gpointer data)
{
	Viewer *v = (Viewer*)data;

	if ((v->cur != NULL) && (v->cur->selection==FALSE))
	{
		v->cur->shown = FALSE;
	}else
	{
		v->main->session->data->hide();
	}
	v->main->gui->update_views();
}
static void dump_cb(GtkWidget *w, gpointer data)
{
	Viewer *v = (Viewer*)data;

	// dump all data inside the current diagram

	v->main->session->data->print();
}

/*
static void network_summary_cb(GtkWidget *w, gpointer data)
{
	Viewer *v = (Viewer*)data;
	Network *n = (Network*)v->cur;
	printf("Network has %d interfaces, %d subnets, %d machines\n",
		n->interfaces.size(), n->subnets.size(), n->machines.size());
}
*/

static gint expose_cb(GtkWidget *w, GdkEventExpose *e, gpointer data)
{
	Viewer *v = (Viewer*)data;
	if (v->canvas!=NULL)
	{
		gdk_draw_pixmap(w->window, w->style->fg_gc[GTK_WIDGET_STATE(w)],
			v->canvas, e->area.x, e->area.y, e->area.x, e->area.y,
			e->area.width, e->area.height);
	}
	return FALSE;
}

static gint configure_cb(GtkWidget *w, GdkEventConfigure *e, gpointer data)
{
	Viewer *v = (Viewer*)data;
	v->draw();
	return TRUE;
}

static void zoom_cb(GtkWidget *w, gpointer data)
{
	Viewer *v = (Viewer*)data;
	GtkAdjustment *adj = (GtkAdjustment*) v->hscale3_adj;
	if (v->scale != (float)adj->value)
	{
//		printf("ZOOM scale=%f, val=%f\n", v->scale, (float)adj->value);
		v->scale = adj->value;
//		v->draw();
		v->refresh = TRUE;
	}
}

static void scroll_cb(GtkWidget *w, gpointer data)
{
	Viewer *v = (Viewer*)data;
	GtkAdjustment *adjx, *adjy;
	adjx = (GtkAdjustment *) v->hscrollbar1_adj;
	adjy = (GtkAdjustment *) v->vscrollbar2_adj;
	if ((v->xpos != (long)adjx->value) || (v->ypos != (long)adjy->value))
	{
//		printf("SCROLL xpos=%ld, xval=%ld, ypos=%ld, yval=%ld\n",
//			v->xpos, (long)adjx->value, v->ypos, (long)adjy->value);
		v->xpos = (long)adjx->value;
		v->ypos = (long)adjy->value;
		v->draw();
	}
}

static gint press_cb(GtkWidget *w, GdkEventButton *e, gpointer data)
{
	((Viewer*)data)->mouse(e->type, (int) e->x, (int) e->y,
		e->state, e->button, e->time);
	return TRUE;
}

static gint motion_cb(GtkWidget *w, GdkEventMotion *e, gpointer data)
{
	((Viewer*)data)->mouse(GDK_MOTION_NOTIFY, (int) e->x, (int) e->y,
		e->state, 0, e->time);
	return TRUE;
}

static gint release_cb(GtkWidget *w, GdkEventButton *e, gpointer data)
{
	((Viewer*)data)->mouse(GDK_BUTTON_RELEASE, (int) e->x, (int) e->y,
		e->state, 0, e->time);
	return TRUE;
}

/*
static void bunch_cb(GtkWidget *w, gpointer data)
{
	Viewer *v = (Viewer*)data;
	v->main->session->data->bunch(v->cur, 1);
	v->draw();
}
*/

static void group_none_cb(GtkWidget *w, gpointer data)
{ Viewer *v = (Viewer*)data; v->group(device_group_none); v->main->gui->update_views(); }
static void group_star_cb(GtkWidget *w, gpointer data)
{ Viewer *v = (Viewer*)data; v->group(device_group_star); v->main->gui->update_views(); }
static void group_bunch_cb(GtkWidget *w, gpointer data)
{ Viewer *v = (Viewer*)data; v->group(device_group_bunch); v->main->gui->update_views(); }
static void bunchall_cb(GtkWidget *w, gpointer data)
{ Viewer *v = (Viewer*)data; v->main->session->data->group_all(device_group_bunch); v->main->gui->update_views(); }
static void starall_cb(GtkWidget *w, gpointer data)
{ Viewer *v = (Viewer*)data; v->main->session->data->group_all(device_group_star); v->main->gui->update_views(); }

static void network_group_bus_cb(GtkWidget *w, gpointer data)
{ Viewer *v = (Viewer*)data; v->netgroup(network_group_bus); }
static void network_group_star_cb(GtkWidget *w, gpointer data)
{ Viewer *v = (Viewer*)data; v->netgroup(network_group_star); }
static void network_group_none_cb(GtkWidget *w, gpointer data)
{ Viewer *v = (Viewer*)data; v->netgroup(network_group_none); }
static void network_group_bunch_cb(GtkWidget *w, gpointer data)
{ Viewer *v = (Viewer*)data; v->netgroup(network_group_bunch); }

static void type_star_cb(GtkWidget *w, gpointer data)
{ Viewer *v = (Viewer*)data; v->settype(network_cloud); v->main->gui->update_views(); }
static void type_hbus_cb(GtkWidget *w, gpointer data)
{ Viewer *v = (Viewer*)data; v->settype(network_hbus); v->main->gui->update_views(); }
static void type_vbus_cb(GtkWidget *w, gpointer data)
{ Viewer *v = (Viewer*)data; v->settype(network_vbus); v->main->gui->update_views(); }
static void type_ptop_cb(GtkWidget *w, gpointer data)
{ Viewer *v = (Viewer*)data; v->settype(network_p2p); v->main->gui->update_views(); }
static void type_fddi_cb(GtkWidget *w, gpointer data)
{ Viewer *v = (Viewer*)data; v->settype(network_ring); v->main->gui->update_views(); }

static void network_position_automatic_cb(GtkWidget *w, gpointer data)
{ Viewer *v = (Viewer*)data; ((Network*)v->cur)->position = network_position_auto;
	v->main->gui->update_views(); }
static void network_position_manual_cb(GtkWidget *w, gpointer data)
{ Viewer *v = (Viewer*)data; ((Network*)v->cur)->position = network_position_manual;
	v->main->gui->update_views(); }

static void telnet_cb(GtkWidget *w, gpointer data)
{
	GError *error;
	gchar *str;
	char buf[256];
	Viewer *v = (Viewer*)data;

	str = gconf_client_get_string (v->main->client,
		CONFIG_SUPPORT_TELNET, &error);

	if ((str!=NULL) && (strcmp(str, "")!=0))
	{
		sprintf(buf, str, ((Device*)v->cur)->name);
		sprintf(buf, "%s &", buf);
		system(buf);
	}

	g_free(str);
}

static void ping_cb(GtkWidget *w, gpointer data)
{
	GError *error;
	gchar *str;
	char buf[256], tip[32];
	Viewer *v = (Viewer*)data;

	if (v->cur->maintype==visual_machine)
		strcpy(tip, ((Machine*)v->cur)->ip.getstring());
	else
		strcpy(tip, ((Device*)v->cur)->pingname.getstring());

	str = gconf_client_get_string (v->main->client,
		CONFIG_SUPPORT_PING, &error);

	if ((str!=NULL) && (strcmp(str, "")!=0))
	{
		sprintf(buf, str, tip);
		sprintf(buf, "%s &", buf);
		system(buf);
	}

	g_free(str);
}

static void center_cb(GtkWidget *w, gpointer data)
{ ((Viewer*)data)->fitToWindow(); }
static void selected_cb(GtkWidget *w, gpointer data)
{ ((Viewer*)data)->fitToSelected(); }
static void show_all_cb(GtkWidget *w, gpointer data)
{ Viewer *v = (Viewer*)data; v->main->session->data->show_all(); v->main->gui->update_views(); }
static void select_all_cb(GtkWidget *w, gpointer data)
{
	Viewer *v = (Viewer*)data;
	v->main->gui->monitor->select_all(TRUE);
	v->main->session->data->select(TRUE);
	v->draw();
}
static void show_all_leaf_cb(GtkWidget *w, gpointer data)
{ ((Viewer*)data)->all_leaf_networks(TRUE); }
static void hide_all_leaf_cb(GtkWidget *w, gpointer data)
{ ((Viewer*)data)->all_leaf_networks(FALSE); }
static void set_default_network_types_cb(GtkWidget *w, gpointer data)
{	((Viewer*)data)->main->session->data->network_type_auto_all();
 	((Viewer*)data)->main->gui->update_views();
}

static void traceroute_cb(GtkWidget *w, gpointer data)
{
	GError *error;
	gchar *str;
	char buf[256];
	Viewer *v = (Viewer*)data;

	str = gconf_client_get_string (v->main->client,
		CONFIG_SUPPORT_TRACEROUTE, &error);

	if ((str!=NULL) && (strcmp(str, "")!=0))
	{
		sprintf(buf, str, ((Device*)v->cur)->name);
		sprintf(buf, "%s &", buf);
		system(buf);
	}

	g_free(str);
}

static void web_cb(GtkWidget *w, gpointer data)
{
	char buf[256];
	Viewer *v = (Viewer*)data;
	sprintf(buf, "http://%s/", ((Device*)v->cur)->name);
	gnome_url_show(buf);
}

static void regrab_cb(GtkWidget *w, gpointer data)
{
	Viewer *v = (Viewer*)data;
	if ((v->cur!=NULL) && (v->cur->selection==FALSE) && (v->cur->maintype==visual_device))
		v->main->scheduler->regrabDevice((Device*)v->cur);
	else
		v->main->scheduler->regrabSelectedDevices();
}

static gint animate_cb(gpointer data)
{ return ((Viewer*)data)->fly(); }

/***************************************************************************
 Viewer Class
 ***************************************************************************/

Viewer::Viewer(GtkWidget *w, Main *m)
{
	window = w;
	main = m;
	scale = 1;
	zoom = 1;
	xpos = 0;
	ypos = 0;
	screw_up_dnd = 0;
	animate_state = 0;
	animate_version = TRUE;
	canvas = NULL;
	refresh = FALSE;
	mode = 0;
	button = 0;

	create();
	popup_menu1();
	popup_menu2();
	popup_menu3();
	popup_menu4();

	alloc_color(orange, 65535, 32767, 32767);

	xgc = gdk_gc_new(window->window);
	gdk_gc_set_function(xgc, GDK_XOR);
	gdk_gc_set_foreground(xgc, &orange);

	diagram = new Diagram(window, main);
}

Viewer::~Viewer()
{
	if (xgc) gdk_gc_unref(xgc);
	if (diagram!=NULL) delete diagram;
}

void Viewer::details_action()
{
	if (cur!=NULL)
	{
		if (cur->maintype==visual_machine)
		{
			main->gui->machinedetails->machine = (Machine*) cur;
			main->gui->machinedetails->show();
		}else
		if (cur->maintype==visual_network)
		{
			main->gui->networkdetails->network = (Network*) cur;
			main->gui->networkdetails->show();
		}else
		if (cur->maintype==visual_device)
		{
			main->gui->devicedetails->device = (Device*) cur;
			main->gui->devicedetails->show();
		}
	}
}

void Viewer::all_leaf_networks(gboolean ch)
{
	Link *ln, *ln2;
	Interface *in;
	Device *r;

	for (ln2=main->session->data->devices.get_head(); ln2!=NULL; ln2=ln2->next)
	{
		r = (Device*) ln2->item;
		for (ln=r->interfaces.get_head(); ln!=NULL; ln=ln->next)
		{
			in = (Interface*) ln->item;
			if ((in->network!=NULL) && (in->network->interfaces.size()==1))
			{
				if ((ch==TRUE) && (r->shown==TRUE))
				{
					in->network->shown = ch;
				}else
				if (ch==FALSE)
				{
					// if hiding network then unselect it
					in->network->shown = ch;
					in->network->selection = ch;
				}
			}
		}
	}
	main->gui->update_views();
}

void Viewer::leaf_networks(gboolean ch)
{
	Link *ln, *ln2;
	Interface *in;
	Device *r;

	if ((cur!=NULL) && (cur->maintype==visual_device) && (cur->selection==FALSE))
	{
		r = (Device*) cur;
		for (ln=r->interfaces.get_head(); ln!=NULL; ln=ln->next)
		{
			in = (Interface*) ln->item;
			if ((in->network!=NULL) && (in->network->interfaces.size()==1))
			{
				in->network->shown = ch;
				// if hiding network then unselect it
				if (ch==FALSE) in->network->selection = ch;
			}
		}
	}else
	for (ln2=main->session->data->devices.get_head(); ln2!=NULL; ln2=ln2->next)
	{
		r = (Device*) ln2->item;
		if (r->selection==TRUE)
		{
			for (ln=r->interfaces.get_head(); ln!=NULL; ln=ln->next)
			{
				in = (Interface*) ln->item;
				if ((in->network!=NULL) &&
						(main->session->data->is_network_a_leaf(in->network)))
				{
					in->network->shown = ch;
					// if hiding network then unselect it
					if (ch==FALSE) in->network->selection = ch;
				}
			}
		}
	}
	main->gui->update_views();
}

// set show/hide for leaf machines

void Viewer::leaf_machines(gboolean ch)
{
	Link *ln, *ln2, *ln3;
	Interface *in;
	Machine *m;
	Network *n;

	if ((cur!=NULL) && (cur->maintype==visual_network) && (cur->selection==FALSE))
	{
		n = (Network*) cur;
		for (ln3=n->interfaces.get_head(); ln3!=NULL; ln3=ln3->next)
		{
			in = (Interface*) ln3->item;
			for (ln=in->machines.get_head(); ln!=NULL; ln=ln->next)
			{
				m = (Machine*) ln->item;
				m->shown = ch;
//				n->position = (ch==FALSE)?network_position_auto:network_position_manual;
				if (ch==FALSE) m->selection = ch;
			}
		}
	}else
	for (ln2=main->session->data->networks.get_head(); ln2!=NULL; ln2=ln2->next)
	{
		n = (Network*) ln2->item;
		if (n->selection==TRUE)
		{
			for (ln3=n->interfaces.get_head(); ln3!=NULL; ln3=ln3->next)
			{
				in = (Interface*) ln3->item;
				for (ln=in->machines.get_head(); ln!=NULL; ln=ln->next)
				{
					m = (Machine*) ln->item;
					m->shown = ch;
//					n->position = (ch==FALSE)?network_position_auto:network_position_manual;
					if (ch==FALSE) m->selection = ch;
				}
			}
		}
	}
	main->gui->update_views();
}

void Viewer::group(DeviceGroupType ch)
{
	Link *ln;
	Device *r;

	if ((cur!=NULL) && (cur->maintype==visual_device) && (cur->selection==FALSE))
	{
		((Device*)cur)->group = ch;
	}else
	for (ln=main->session->data->devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		if (r->selection==TRUE) r->group = ch;
	}
	main->gui->update_views();
}

void Viewer::netgroup(NetworkGroupType ch)
{
	Link *ln;
	Network *n;

	if ((cur!=NULL) && (cur->maintype==visual_network) &&
			(cur->selection==FALSE))
	{
		((Network*)cur)->group = ch;
	}else
	for (ln=main->session->data->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->selection==TRUE) n->group = ch;
	}
	main->gui->update_views();
}

void Viewer::settype(NetworkType ch)
{
	Link *ln;
	Network *n;

	if ((cur!=NULL) && (cur->maintype==visual_network) && (cur->selection==FALSE))
	{
		main->session->data->select(FALSE);
		if (ch==network_p2p)
		{
			if (((Network*)cur)->interfaces.size()==2)
			{
				((Network*)cur)->subtype = network_p2p;
				cur->w = 96; cur->h = 96;
			}
		}else
		{
			((Network*)cur)->subtype = ch;
			if (ch==network_ring)
			{ cur->w = 96*4; cur->h = 96*4; }else
			if ((ch!=network_hbus) && (ch!=network_vbus))
			{ cur->w = 96; cur->h = 96; }
		}
		cur->selection = TRUE;
	}else
	for (ln=main->session->data->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->selection==TRUE) 
		{
			if (ch==network_p2p)
			{
				if (n->interfaces.size()==2)
				{
					n->subtype = network_p2p;
					n->w = 96; n->h = 96;
				}
			}else
			{
				n->subtype = ch;
				if (ch==network_ring)
				{ n->w = 96*4; n->h = 96*4; }else
				if ((ch!=network_hbus) && (ch!=network_vbus))
				{ n->w = 96; n->h = 96; }
			}
		}
	}
}

void Viewer::create()
{
	pixmap1 = create_pixmap(window, sfitall_xpm);
	pixmap2 = create_pixmap(window, sfitsel_xpm);
	pixmap3 = create_pixmap(window, sshowall_xpm);

  vbox3 = gtk_vbox_new (FALSE, 2);
  gtk_widget_show (vbox3);
  gtk_widget_set_usize (vbox3, 240, -2);
  gtk_container_set_border_width (GTK_CONTAINER (vbox3), 2);

  hbox7 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox7);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox7, FALSE, TRUE, 0);

  button1 = gtk_button_new ();
  gtk_widget_show (button1);
	gtk_container_add (GTK_CONTAINER (button1), pixmap1);
  gtk_box_pack_start (GTK_BOX (hbox7), button1, FALSE, FALSE, 0);
	gtk_tooltips_set_tip(gtk_tooltips_new(), button1, "Fit to Window", NULL);
	gtk_signal_connect(GTK_OBJECT(button1), "pressed",
		GTK_SIGNAL_FUNC(center_cb), this);

  button2 = gtk_button_new ();
  gtk_widget_show (button2);
	gtk_container_add (GTK_CONTAINER (button2), pixmap2);
  gtk_box_pack_start (GTK_BOX (hbox7), button2, FALSE, FALSE, 0);
	gtk_tooltips_set_tip(gtk_tooltips_new(), button2, "Fit to Selected", NULL);
  gtk_signal_connect(GTK_OBJECT(button2), "pressed",
		GTK_SIGNAL_FUNC(selected_cb), this);

  button3 = gtk_button_new ();
  gtk_widget_show (button3);
	gtk_container_add (GTK_CONTAINER (button3), pixmap3);
  gtk_box_pack_start (GTK_BOX (hbox7), button3, FALSE, FALSE, 0);
	gtk_tooltips_set_tip(gtk_tooltips_new(), button3, "Show All", NULL);
  gtk_signal_connect(GTK_OBJECT(button3), "pressed",
		GTK_SIGNAL_FUNC(show_all_cb), this);

  label8 = gtk_label_new ("Zoom");
  gtk_widget_show (label8);
  gtk_box_pack_start (GTK_BOX (hbox7), label8, FALSE, FALSE, 0);
//  gtk_widget_set_usize (label8, 60, -2);

  hscale3_adj = gtk_adjustment_new (1, 1, 10, 0.0001, 0.0001, 0);
  hscale3 = gtk_hscale_new (GTK_ADJUSTMENT (hscale3_adj));
  gtk_widget_show (hscale3);
  gtk_box_pack_start (GTK_BOX (hbox7), hscale3, TRUE, TRUE, 0);
  gtk_scale_set_digits (GTK_SCALE (hscale3), 4);
  gtk_scale_set_draw_value (GTK_SCALE (hscale3), FALSE);
	gtk_signal_connect (GTK_OBJECT (hscale3_adj), "value_changed",
		GTK_SIGNAL_FUNC (zoom_cb), this);

  label8 = gtk_label_new ("Quick Search");
  gtk_widget_show (label8);
  gtk_box_pack_start (GTK_BOX (hbox7), label8, FALSE, FALSE, 0);
//  gtk_widget_set_usize (label8, 60, -2);

	quicksearch = gtk_entry_new ();
  gtk_widget_show (quicksearch);
  gtk_box_pack_start (GTK_BOX (hbox7), quicksearch, TRUE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(quicksearch), "activate",
		GTK_SIGNAL_FUNC(quicksearch_cb), this);

  table2 = gtk_table_new (2, 2, FALSE);
  gtk_widget_show (table2);
  gtk_box_pack_start (GTK_BOX (vbox3), table2, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (table2), 2);
  gtk_table_set_row_spacings (GTK_TABLE (table2), 2);
  gtk_table_set_col_spacings (GTK_TABLE (table2), 2);

  viewport1 = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (viewport1);
  gtk_table_attach (GTK_TABLE (table2), viewport1, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  drawingarea1 = gtk_drawing_area_new ();
	gtk_widget_set_events(drawingarea1, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK );
  gtk_widget_show (drawingarea1);
  gtk_container_add (GTK_CONTAINER (viewport1), drawingarea1);
	gtk_signal_connect (GTK_OBJECT (drawingarea1), "expose_event",
		(GtkSignalFunc) expose_cb, this);
	gtk_signal_connect (GTK_OBJECT(drawingarea1),"configure_event",
		(GtkSignalFunc) configure_cb, this);
	gtk_signal_connect (GTK_OBJECT (drawingarea1), "motion_notify_event",
		(GtkSignalFunc) motion_cb, this);
	gtk_signal_connect (GTK_OBJECT (drawingarea1), "button_press_event",
		(GtkSignalFunc) press_cb, this);
	gtk_signal_connect (GTK_OBJECT (drawingarea1), "button_release_event",
		(GtkSignalFunc) release_cb, this);
	GtkTooltips *tooltips = gtk_tooltips_new ();
	gtk_tooltips_set_tip (tooltips, drawingarea1, "This is button 1", NULL);

	vscrollbar2_adj = gtk_adjustment_new (0, 0, 100, 1, 1, 1);
  vscrollbar2 = gtk_vscrollbar_new (GTK_ADJUSTMENT (vscrollbar2_adj));
  gtk_widget_show (vscrollbar2);
  gtk_table_attach (GTK_TABLE (table2), vscrollbar2, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_signal_connect (GTK_OBJECT (vscrollbar2_adj), "value_changed",
		GTK_SIGNAL_FUNC (scroll_cb), this);

	hscrollbar1_adj = gtk_adjustment_new (0, 0, 100, 1, 1, 1);
  hscrollbar1 = gtk_hscrollbar_new (GTK_ADJUSTMENT (hscrollbar1_adj));
  gtk_widget_show (hscrollbar1);
  gtk_table_attach (GTK_TABLE (table2), hscrollbar1, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_signal_connect (GTK_OBJECT (hscrollbar1_adj), "value_changed",
		GTK_SIGNAL_FUNC (scroll_cb), this);
}

void Viewer::popup_menu1()	// Devices
{
	GSList *group_group = NULL;
	int i;

	popup = gtk_menu_new();

	dmn[0] = gtk_menu_item_new_with_label ("Properties");
	dmn[1] = gtk_menu_item_new_with_label ("Hide");
	dmn[2] = gtk_menu_item_new_with_label ("Merge");
	dmn[3] = gtk_menu_item_new_with_label ("Split");
	dmn[4] = gtk_menu_item_new_with_label ("Delete");

	dmn[5] = gtk_menu_item_new_with_label ("Graph");
	dmn[6] = gtk_menu_item_new_with_label ("Regrab");
	dmn[7] = gtk_menu_item_new_with_label ("Helper Apps");
	dmn[8] = gtk_menu_item_new ();
	dmn[9] = gtk_menu_item_new_with_label ("Show Leaf Networks");
	dmn[10] = gtk_menu_item_new_with_label ("Hide Leaf Networks");
	dmn[11] = gtk_menu_item_new_with_label ("Group Leaf Networks");

	for (i=0; i<=11; i++)
	{
		gtk_widget_show(dmn[i]);
		gtk_container_add(GTK_CONTAINER(popup), dmn[i]);
	}

	gtk_signal_connect(GTK_OBJECT(dmn[0]), "activate",
		GTK_SIGNAL_FUNC(details_cb), this);
	gtk_signal_connect(GTK_OBJECT(dmn[1]), "activate",
		GTK_SIGNAL_FUNC(hide_cb), this);
	gtk_signal_connect(GTK_OBJECT(dmn[2]), "activate",
		GTK_SIGNAL_FUNC(merge_cb), this);
	gtk_signal_connect(GTK_OBJECT(dmn[3]), "activate",
		GTK_SIGNAL_FUNC(split_cb), this);
	gtk_signal_connect(GTK_OBJECT(dmn[4]), "activate",
		GTK_SIGNAL_FUNC(delete_cb), this);
	gtk_signal_connect(GTK_OBJECT(dmn[5]), "activate",
		GTK_SIGNAL_FUNC(device_graph_cb), this);
	gtk_signal_connect(GTK_OBJECT(dmn[6]), "activate",
		GTK_SIGNAL_FUNC(regrab_cb), this);
	gtk_signal_connect(GTK_OBJECT(dmn[9]), "activate",
		GTK_SIGNAL_FUNC(show_leaf_networks_cb), this);
	gtk_signal_connect(GTK_OBJECT(dmn[10]), "activate",
		GTK_SIGNAL_FUNC(hide_leaf_networks_cb), this);

/////////////////////////////////////////////////

	dmn[30] = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(dmn[7]), dmn[30]);
	dmn[31] = gtk_menu_item_new_with_label ("Ping");
	dmn[32] = gtk_menu_item_new_with_label ("Telnet");
	dmn[33] = gtk_menu_item_new_with_label ("Traceroute");
	dmn[34] = gtk_menu_item_new_with_label ("Web Access");

	for (i=31; i<=34; i++)
	{
		gtk_widget_show(dmn[i]);
		gtk_container_add(GTK_CONTAINER(dmn[30]), dmn[i]);
	}
	gtk_signal_connect(GTK_OBJECT(dmn[31]), "activate",
		GTK_SIGNAL_FUNC(ping_cb), this);
	gtk_signal_connect(GTK_OBJECT(dmn[32]), "activate",
		GTK_SIGNAL_FUNC(telnet_cb), this);
	gtk_signal_connect(GTK_OBJECT(dmn[33]), "activate",
		GTK_SIGNAL_FUNC(traceroute_cb), this);
	gtk_signal_connect(GTK_OBJECT(dmn[34]), "activate",
		GTK_SIGNAL_FUNC(web_cb), this);

///////////////////////////////////////

	dmn[20] = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(dmn[11]), dmn[20]);
	dmn[21] = gtk_radio_menu_item_new_with_label (group_group, "Bunch");
	group_group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (dmn[21]));
	dmn[22] = gtk_radio_menu_item_new_with_label (group_group, "Star");
	group_group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (dmn[22]));
	dmn[23] = gtk_radio_menu_item_new_with_label (group_group, "None");
	group_group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (dmn[23]));

	for (i=21; i<=23; i++)
	{
		gtk_widget_show(dmn[i]);
		gtk_container_add(GTK_CONTAINER(dmn[20]), dmn[i]);
	}

	gtk_signal_connect(GTK_OBJECT(dmn[23]), "activate",
		GTK_SIGNAL_FUNC(group_none_cb), this);
	gtk_signal_connect(GTK_OBJECT(dmn[22]), "activate",
		GTK_SIGNAL_FUNC(group_star_cb), this);
	gtk_signal_connect(GTK_OBJECT(dmn[21]), "activate",
		GTK_SIGNAL_FUNC(group_bunch_cb), this);
}

void Viewer::popup_menu2()
{
	GSList *type_group = NULL;
	GSList *position_group = NULL;
	GSList *machine_group = NULL;
	int i;

	popup2 = gtk_menu_new();

	nmn[0] = gtk_menu_item_new_with_label ("Properties");
	nmn[1] = gtk_menu_item_new_with_label ("Hide");
//	nmn[2] = gtk_menu_item_new_with_label ("Graph");
	nmn[2] = gtk_menu_item_new_with_label ("Type");
	nmn[3] = gtk_menu_item_new_with_label ("Position");
	nmn[4] = gtk_menu_item_new ();
	nmn[5] = gtk_menu_item_new_with_label ("Show Machines");
	nmn[6] = gtk_menu_item_new_with_label ("Hide Machines");
	nmn[7] = gtk_menu_item_new_with_label ("Group Machines");
	for (i=0; i<=7; i++)
	{
		gtk_widget_show(nmn[i]);
		gtk_container_add(GTK_CONTAINER(popup2), nmn[i]);
	}
	gtk_signal_connect(GTK_OBJECT(nmn[0]), "activate",
		GTK_SIGNAL_FUNC(details_cb), this);
	gtk_signal_connect(GTK_OBJECT(nmn[1]), "activate",
		GTK_SIGNAL_FUNC(hide_cb), this);
//	gtk_signal_connect(GTK_OBJECT(nmn[2]), "activate",
//		GTK_SIGNAL_FUNC(network_graph_cb), this);
	gtk_signal_connect(GTK_OBJECT(nmn[5]), "activate",
		GTK_SIGNAL_FUNC(show_leaf_machines_cb), this);
	gtk_signal_connect(GTK_OBJECT(nmn[6]), "activate",
		GTK_SIGNAL_FUNC(hide_leaf_machines_cb), this);

////////////////////////////////////////////////////

	nmn[10] = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(nmn[7]), nmn[10]);
	nmn[11] = gtk_radio_menu_item_new_with_label (machine_group, "Bus [FIXME]");
	type_group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (nmn[11]));
	nmn[12] = gtk_radio_menu_item_new_with_label (machine_group, "Star [FIXME]");
	type_group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (nmn[12]));
	nmn[13] = gtk_radio_menu_item_new_with_label (machine_group, "None [FIXME]");
	type_group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (nmn[13]));
	nmn[14] = gtk_radio_menu_item_new_with_label (machine_group, "Bunch FIXME");
	type_group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (nmn[14]));
	for (i=11; i<=14; i++)
	{
		gtk_widget_show(nmn[i]);
		gtk_container_add(GTK_CONTAINER(nmn[10]), nmn[i]);
	}
	gtk_signal_connect(GTK_OBJECT(nmn[11]), "activate",
		GTK_SIGNAL_FUNC(network_group_bus_cb), this);
	gtk_signal_connect(GTK_OBJECT(nmn[12]), "activate",
		GTK_SIGNAL_FUNC(network_group_star_cb), this);
	gtk_signal_connect(GTK_OBJECT(nmn[13]), "activate",
		GTK_SIGNAL_FUNC(network_group_none_cb), this);
	gtk_signal_connect(GTK_OBJECT(nmn[14]), "activate",
		GTK_SIGNAL_FUNC(network_group_bunch_cb), this);

////////////////////////////////////////////////////

	nmn[15] = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(nmn[3]), nmn[15]);
	nmn[16] = gtk_radio_menu_item_new_with_label (position_group, "Automatic");
	type_group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (nmn[16]));
	nmn[17] = gtk_radio_menu_item_new_with_label (position_group, "Manual");
	type_group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (nmn[17]));
	for (i=16; i<=17; i++)
	{
		gtk_widget_show(nmn[i]);
		gtk_container_add(GTK_CONTAINER(nmn[15]), nmn[i]);
	}
	gtk_signal_connect(GTK_OBJECT(nmn[16]), "activate",
		GTK_SIGNAL_FUNC(network_position_automatic_cb), this);
	gtk_signal_connect(GTK_OBJECT(nmn[17]), "activate",
		GTK_SIGNAL_FUNC(network_position_manual_cb), this);

////////////////////////////////////////////////////

	nmn[20] = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(nmn[2]), nmn[20]);
	nmn[21] = gtk_radio_menu_item_new_with_label (type_group, "Star");
	type_group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (nmn[21]));
	nmn[22] = gtk_radio_menu_item_new_with_label (type_group, "H Bus");
	type_group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (nmn[22]));
	nmn[23] = gtk_radio_menu_item_new_with_label (type_group, "V Bus");
	type_group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (nmn[23]));
	nmn[24] = gtk_radio_menu_item_new_with_label (type_group, "P-to-P");
	type_group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (nmn[24]));
	nmn[25] = gtk_radio_menu_item_new_with_label (type_group, "FDDI");
	type_group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (nmn[25]));
	for (i=21; i<=25; i++)
	{
		gtk_widget_show(nmn[i]);
		gtk_container_add(GTK_CONTAINER(nmn[20]), nmn[i]);
	}
	gtk_signal_connect(GTK_OBJECT(nmn[21]), "activate",
		GTK_SIGNAL_FUNC(type_star_cb), this);
	gtk_signal_connect(GTK_OBJECT(nmn[22]), "activate",
		GTK_SIGNAL_FUNC(type_hbus_cb), this);
	gtk_signal_connect(GTK_OBJECT(nmn[23]), "activate",
		GTK_SIGNAL_FUNC(type_vbus_cb), this);
	gtk_signal_connect(GTK_OBJECT(nmn[24]), "activate",
		GTK_SIGNAL_FUNC(type_ptop_cb), this);
	gtk_signal_connect(GTK_OBJECT(nmn[25]), "activate",
		GTK_SIGNAL_FUNC(type_fddi_cb), this);
}

void Viewer::popup_menu3()
{
	int i;

	popup3 = gtk_menu_new();
	mmn[0] = gtk_menu_item_new_with_label ("Properties");
	mmn[1] = gtk_menu_item_new_with_label ("Hide");
	mmn[2] = gtk_menu_item_new_with_label ("Helper Apps");
	for (i=0; i<=2; i++)
	{
		gtk_widget_show(mmn[i]);
		gtk_container_add(GTK_CONTAINER(popup3), mmn[i]);
	}
  gtk_signal_connect(GTK_OBJECT(mmn[0]), "activate",
		GTK_SIGNAL_FUNC(details_cb), this);
  gtk_signal_connect(GTK_OBJECT(mmn[1]), "activate",
		GTK_SIGNAL_FUNC(hide_cb), this);

////////////////////////////////////////////////

	mmn[10] = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(mmn[2]), mmn[10]);
	mmn[11] = gtk_menu_item_new_with_label ("Ping");
//	mmn[12] = gtk_menu_item_new_with_label ("Telnet");
//	mmn[13] = gtk_menu_item_new_with_label ("Traceroute");
//	mmn[14] = gtk_menu_item_new_with_label ("Web Access");
	for (i=11; i<=11; i++)
	{
		gtk_widget_show(mmn[i]);
		gtk_container_add(GTK_CONTAINER(mmn[10]), mmn[i]);
	}
	gtk_signal_connect(GTK_OBJECT(mmn[11]), "activate",
		GTK_SIGNAL_FUNC(ping_cb), this);
//	gtk_signal_connect(GTK_OBJECT(mmn[12]), "activate",
//		GTK_SIGNAL_FUNC(telnet_cb), this);
//	gtk_signal_connect(GTK_OBJECT(mmn[13]), "activate",
//		GTK_SIGNAL_FUNC(traceroute_cb), this);
//	gtk_signal_connect(GTK_OBJECT(mmn[14]), "activate",
//		GTK_SIGNAL_FUNC(web_cb), this);
}

void Viewer::popup_menu4()	// Background
{
	GtkWidget *mn[13];
	int i;

	popup4 = gtk_menu_new();
	mn[0] = gtk_menu_item_new_with_label ("Fit to Window");
	mn[1] = gtk_menu_item_new_with_label ("Fit to Selected");
	mn[2] = gtk_menu_item_new_with_label ("Select All");
	mn[3] = gtk_menu_item_new ();
	mn[4] = gtk_menu_item_new_with_label ("Bunch All Networks");
	mn[5] = gtk_menu_item_new_with_label ("Star All Networks");
	mn[6] = gtk_menu_item_new ();
	mn[7] = gtk_menu_item_new_with_label ("Show All Devices");
	mn[8] = gtk_menu_item_new_with_label ("Show All Leaf Networks");
	mn[9] = gtk_menu_item_new_with_label ("Hide All Leaf Networks");
	mn[10] = gtk_menu_item_new_with_label ("Apply Default Network Types");
	mn[11] = gtk_menu_item_new ();
	mn[12] = gtk_menu_item_new_with_label ("Test 1");
	for (i=0; i<=12; i++)
	{
		gtk_widget_show(mn[i]);
		gtk_container_add(GTK_CONTAINER(popup4), mn[i]);
	}

  gtk_signal_connect(GTK_OBJECT(mn[0]), "activate",
		GTK_SIGNAL_FUNC(center_cb), this);
	gtk_signal_connect(GTK_OBJECT(mn[1]), "activate",
		GTK_SIGNAL_FUNC(selected_cb), this);
	gtk_signal_connect(GTK_OBJECT(mn[2]), "activate",
		GTK_SIGNAL_FUNC(select_all_cb), this);
	gtk_signal_connect(GTK_OBJECT(mn[4]), "activate",
		GTK_SIGNAL_FUNC(bunchall_cb), this);
	gtk_signal_connect(GTK_OBJECT(mn[5]), "activate",
		GTK_SIGNAL_FUNC(starall_cb), this);
	gtk_signal_connect(GTK_OBJECT(mn[7]), "activate",
		GTK_SIGNAL_FUNC(show_all_cb), this);
	gtk_signal_connect(GTK_OBJECT(mn[8]), "activate",
		GTK_SIGNAL_FUNC(show_all_leaf_cb), this);
	gtk_signal_connect(GTK_OBJECT(mn[9]), "activate",
		GTK_SIGNAL_FUNC(hide_all_leaf_cb), this);
	gtk_signal_connect(GTK_OBJECT(mn[10]), "activate",
		GTK_SIGNAL_FUNC(set_default_network_types_cb), this);
	gtk_signal_connect(GTK_OBJECT(mn[12]), "activate",
		GTK_SIGNAL_FUNC(dump_cb), this);
}

void Viewer::xbox(Visual *v)
{
	gdk_draw_rectangle(drawingarea1->window, xgc, FALSE,
		int(v->dx-(v->w/(2*scale))), int(v->dy-(v->h/(2*scale))),
		int(v->w/scale), int(v->h/scale));
//	gdk_draw_rectangle(drawingarea1->window, xgc, FALSE,
//		int(v->rx-(v->w/(2*scale))), int(v->ry-(v->h/(2*scale))),
//		int(v->w/scale), int(v->h/scale));
}

void Viewer::scroll_wheel(int dir)
{
	float val;
	GtkAdjustment *adj;

//	printf("ZOOM1 %d\n", dir);
	adj = (GtkAdjustment*) hscale3_adj;
	val = ((float)adj->value) + (4.5) - dir;
	if ((val<0) || (val>zoom)) return;
//	printf("ZOOM2 %d\n", dir);
	gtk_adjustment_set_value(adj, ((float)adj->value) + (4.5) - dir);
}

void Viewer::mouse(GdkEventType type, int x, int y, int s, int btn, guint32 t)
{
	Link *ln;
	Device *r;
	Network *n;
	Machine *m;
	GtkWidget *w;
	int i, j;
	gboolean ready_to_merge;

	long orgx, orgy, ab1, ab2, ab3, ab4;

	if (main->session->data==NULL) return;

	if ((btn==4) || (btn==5)) { scroll_wheel(btn); return; }

	ready_to_merge = main->session->data->readyToMerge();

	if ((type == GDK_BUTTON_PRESS) || (type == GDK_2BUTTON_PRESS))
	{
		screw_up_dnd = 0;
		gdk_pointer_grab (drawingarea1->window, FALSE,
			(GdkEventMask) (GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK),
			NULL, NULL, t);
		button = btn;
		mode = 0;

//		printf("%ld, %ld\n", main->session->data->minx, main->session->data->miny);
//		printf("%ld, %ld\n", xpos, ypos);

		orgx = (long) (GTK_ADJUSTMENT(hscrollbar1_adj)->value+
			((x-(width/2))*scale));
		orgy = (long) (GTK_ADJUSTMENT(vscrollbar2_adj)->value+
			((y-(height/2))*scale));

		for (ln = main->session->data->devices.get_tail(); ln!=NULL; ln=ln->prev)
		{
			r = (Device*) ln->item;
			if ((r->shown==TRUE) &&
					(r->x-(r->w/2)<=orgx) && (r->x+(r->w/2)>=orgx) &&
					(r->y-(r->h/2)<=orgy) && (r->y+(r->h/2)>=orgy))
			{
				if (mode == 0) cur = r;
				mode = 1;
				if (s & GDK_SHIFT_MASK) mode = 2;
				if (s & GDK_CONTROL_MASK) mode = 2;
			}
		}

		for (ln = main->session->data->machines.get_tail(); ln!=NULL; ln=ln->prev)
		{
			m = (Machine*) ln->item;
			if ((m->shown==TRUE) && (m->interface->network->shown==TRUE) &&
					(m->x-(m->w/2)<=orgx) && (m->x+(m->w/2)>=orgx) &&
					(m->y-(m->h/2)<=orgy) && (m->y+(m->h/2)>=orgy))
			{
				if (mode == 0) cur = m;
				mode = 1;
				if (s & GDK_SHIFT_MASK) mode = 2;
				if (s & GDK_CONTROL_MASK) mode = 2;
			}
		}

		for (ln = main->session->data->networks.get_tail(); ln!=NULL; ln=ln->prev)
		{
			n = (Network*) ln->item;
			if ((n->shown==TRUE) &&
					(n->x-(n->w/2)<=orgx) && (n->x+(n->w/2)>=orgx) &&
					(n->y-(n->h/2)<=orgy) && (n->y+(n->h/2)>=orgy))
			{
				if (mode == 0) cur = n;
				mode = 1;
				if (s & GDK_SHIFT_MASK) mode = 2;
				if (s & GDK_CONTROL_MASK) mode = 2;
			}
		}

		// bounding box
		if ((button == 1) && (mode == 0))
		{
			obx = bbx = x;
			oby = bby = y;
			draw_bb(drawingarea1->window);
		}else
		// properties window
		if ((button == 1) && (mode == 1) && (type == GDK_2BUTTON_PRESS))
		{
			details_action();
		}else
		// drag object
		if ((button == 1) && (mode == 1) && (type == GDK_BUTTON_PRESS))
		{
			nx = ox = x;
			ny = oy = y;
			if (cur->selection==FALSE)
			{
				// move current
//				cur->fix = 1;
				// we need to update rx,ry for main viewer so mouse clicks work
				cur->rx = (cur->x/scale)+(width/2)-(xpos/scale);
				cur->ry = (cur->y/scale)+(height/2)-(ypos/scale);
				cur->dx = (cur->x/scale)+(width/2)-(xpos/scale);
				cur->dy = (cur->y/scale)+(height/2)-(ypos/scale);
				xbox(cur);
			}else
			{
				// move all selected objects
				for (ln = main->session->data->devices.get_head(); ln!=NULL; ln=ln->next)
				{
					r = (Device*) ln->item;
					if ((r->shown==TRUE) && (r->selection==TRUE))
					{
//						r->fix = '1';
						r->rx = (r->x/scale)+(width/2)-(xpos/scale);
						r->ry = (r->y/scale)+(height/2)-(ypos/scale);
						r->dx = (r->x/scale)+(width/2)-(xpos/scale);
						r->dy = (r->y/scale)+(height/2)-(ypos/scale);
						xbox(r);
					}
				}
				for (ln = main->session->data->machines.get_head(); ln!=NULL; ln=ln->next)
				{
					m = (Machine*) ln->item;
					if ((m->shown==TRUE) && (m->selection==TRUE) &&
							(m->interface->network->shown==TRUE))
					{
//						m->fix = '1';
						m->rx = (m->x/scale)+(width/2)-(xpos/scale);
						m->ry = (m->y/scale)+(height/2)-(ypos/scale);
						m->dx = (m->x/scale)+(width/2)-(xpos/scale);
						m->dy = (m->y/scale)+(height/2)-(ypos/scale);
						xbox(m);
					}
				}
				for (ln = main->session->data->networks.get_head(); ln!=NULL; ln=ln->next)
				{
					n = (Network*) ln->item;
					if ((n->shown==TRUE) && (n->selection==TRUE))
					{
//						n->fix = '1';
						n->rx = (n->x/scale)+(width/2)-(xpos/scale);
						n->ry = (n->y/scale)+(height/2)-(ypos/scale);
						n->dx = (n->x/scale)+(width/2)-(xpos/scale);
						n->dy = (n->y/scale)+(height/2)-(ypos/scale);
						xbox(n);
					}
				}
			}
		}else
		// popup menu
		if ((button == 3) && (mode == 1))
		{
     	if (cur->maintype==visual_device)
			{
				r = (Device*) cur;
				// don't use gtk_check_menu_item_set_active as we
				// don't want to create an activate event
				GTK_CHECK_MENU_ITEM(dmn[21])->active = FALSE;
				GTK_CHECK_MENU_ITEM(dmn[22])->active = FALSE;
				GTK_CHECK_MENU_ITEM(dmn[23])->active = FALSE;
				if (r->group==device_group_bunch) w = dmn[21]; else
				if (r->group==device_group_star) w = dmn[22]; else w = dmn[23];
				GTK_CHECK_MENU_ITEM(w)->active = TRUE;

				// split & merge menu items
				gtk_widget_set_sensitive(dmn[3], (r->subtype==device_merged)?TRUE:FALSE);
				gtk_widget_set_sensitive(dmn[2],
					((ready_to_merge)&&(r->selection==TRUE))?TRUE:FALSE);

				gtk_menu_popup(GTK_MENU(popup), NULL, NULL, NULL, NULL, button, 0);
			}else
			if (cur->maintype==visual_network)
			{
				n = (Network*) cur;
				// don't use gtk_check_menu_item_set_active as we
				// don't want to create an activate event
				GTK_CHECK_MENU_ITEM(nmn[21])->active = FALSE;
				GTK_CHECK_MENU_ITEM(nmn[22])->active = FALSE;
				GTK_CHECK_MENU_ITEM(nmn[23])->active = FALSE;
				GTK_CHECK_MENU_ITEM(nmn[24])->active = FALSE;
				GTK_CHECK_MENU_ITEM(nmn[25])->active = FALSE;
				if (n->subtype==network_hbus) w = nmn[22]; else
				if (n->subtype==network_vbus) w = nmn[23]; else
				if (n->subtype==network_p2p)  w = nmn[24]; else
				if (n->subtype==network_ring) w = nmn[25]; else w = nmn[21];
				GTK_CHECK_MENU_ITEM(w)->active = TRUE;

				// gray out FDDI & P2P if network can't be set as such
				gtk_widget_set_sensitive(nmn[24],
					((n->numMachines()==0) && (n->interfaces.size()==2))?TRUE:FALSE);
				gtk_widget_set_sensitive(nmn[25], n->matchFDDI());

				if (n->subtype==network_p2p)
				{
					// gray out manual option if a p2p network type
					GTK_CHECK_MENU_ITEM(nmn[16])->active = TRUE;
					GTK_CHECK_MENU_ITEM(nmn[17])->active = FALSE;
					gtk_widget_set_sensitive(nmn[17], FALSE);
				}else
				{
					GTK_CHECK_MENU_ITEM(nmn[16])->active = FALSE;
					GTK_CHECK_MENU_ITEM(nmn[17])->active = FALSE;
					if (n->position==network_position_auto)
						w = nmn[16]; else w = nmn[17];
					GTK_CHECK_MENU_ITEM(w)->active = TRUE;
					gtk_widget_set_sensitive(nmn[17], TRUE);
				}

				GTK_CHECK_MENU_ITEM(nmn[11])->active = FALSE;
				GTK_CHECK_MENU_ITEM(nmn[12])->active = FALSE;
				GTK_CHECK_MENU_ITEM(nmn[13])->active = FALSE;
				GTK_CHECK_MENU_ITEM(nmn[14])->active = FALSE;
				if (n->group==network_group_bus) w = nmn[11]; else
				if (n->group==network_group_star) w = nmn[12]; else
				if (n->group==network_group_bunch) w = nmn[14]; else w = nmn[13];
				GTK_CHECK_MENU_ITEM(w)->active = TRUE;

				gtk_menu_popup(GTK_MENU(popup2), NULL, NULL, NULL, NULL, button, 0);
			}else
			if (cur->maintype==visual_machine)
			{
				gtk_menu_popup(GTK_MENU(popup3), NULL, NULL, NULL, NULL, button, 0);
			}
		}else
		if ((button == 3) && (mode == 0))
		{
			gtk_menu_popup(GTK_MENU(popup4), NULL, NULL, NULL, NULL, button, 0);
		}
	}else
	if (type == GDK_MOTION_NOTIFY)
	{
		if ((button == 1) && (mode == 0))
		{
			if (screw_up_dnd == 1) screw_up_dnd = 0; else
				draw_bb(drawingarea1->window);
			bbx = x;
			bby = y;
			draw_bb(drawingarea1->window);
		}else
		if ((button == 1) && (mode == 1))
		{
			// move all selected objects
			if (cur==NULL) ;	// skip
			else
			if (cur->selection==FALSE)
			{
				if (screw_up_dnd == 1) screw_up_dnd = 0; else
					xbox(cur);
				cur->dx = cur->dx+((x-ox));
				cur->dy = cur->dy+((y-oy));
				cur->rx = cur->rx+((x-ox));
				cur->ry = cur->ry+((y-oy));
				xbox(cur);
			}else
			{
				for (ln = main->session->data->devices.get_head(); ln!=NULL; ln=ln->next)
				{
					r = (Device*) ln->item;
					if (r->selection==TRUE)
					{
						if (screw_up_dnd == 1) screw_up_dnd = 0; else
							xbox(r);
						r->dx = r->dx+((x-ox));
						r->dy = r->dy+((y-oy));
						r->rx = r->rx+((x-ox));
						r->ry = r->ry+((y-oy));
						xbox(r);
					}
				}
				for (ln = main->session->data->machines.get_head(); ln!=NULL; ln=ln->next)
				{
					m = (Machine*) ln->item;
					if (m->selection==TRUE)
					{
						if (screw_up_dnd == 1) screw_up_dnd = 0; else
							xbox(m);
						m->dx = m->dx+((x-ox));
						m->dy = m->dy+((y-oy));
						m->rx = m->rx+((x-ox));
						m->ry = m->ry+((y-oy));
						xbox(m);
					}
				}
				for (ln = main->session->data->networks.get_head(); ln!=NULL; ln=ln->next)
				{
					n = (Network*) ln->item;
					if (n->selection==TRUE)
					{
						if (screw_up_dnd == 1) screw_up_dnd = 0; else
							xbox(n);
						n->dx = n->dx+((x-ox));
						n->dy = n->dy+((y-oy));
						n->rx = n->rx+((x-ox));
						n->ry = n->ry+((y-oy));
						xbox(n);
					}
				}
			}
			ox = x;
			oy = y;
		}
	}else
	if (type == GDK_BUTTON_RELEASE)
	{
		orgx = (long) (GTK_ADJUSTMENT(hscrollbar1_adj)->value+
			((x-(width/2))*scale));
		orgy = (long) (GTK_ADJUSTMENT(vscrollbar2_adj)->value+
			((y-(height/2))*scale));

		if ((button == 1) && (mode == 0))
		{
			// if we are not shift clicking or ctrl clicking
			// we must deselect everything first
			if (!  ((s & GDK_SHIFT_MASK) || (s & GDK_CONTROL_MASK)))

			{
				main->gui->monitor->select_all(FALSE);
				main->session->data->select(FALSE);
			}

			// make sure obx,oby is the top left corner
			if (obx>bbx) { i = bbx; bbx = obx; obx = i; }
			if (oby>bby) { i = bby; bby = oby; oby = i; }

			ab1 = (long) (GTK_ADJUSTMENT(hscrollbar1_adj)->value+
				((obx-(width/2))*scale));
			ab2 = (long) (GTK_ADJUSTMENT(vscrollbar2_adj)->value+
				((oby-(height/2))*scale));
			ab3 = (long) (GTK_ADJUSTMENT(hscrollbar1_adj)->value+
				((bbx-(width/2))*scale));
			ab4 = (long) (GTK_ADJUSTMENT(vscrollbar2_adj)->value+
				((bby-(height/2))*scale));

			// select those in the bounding box
			for (ln = main->session->data->devices.get_head(); ln!=NULL; ln=ln->next)
			{
				r = (Device*) ln->item;
				if ((r->shown==TRUE) &&
						(r->x-(r->w/2)>=ab1) && (r->x+(r->w/2)<=ab3) &&
						(r->y-(r->h/2)>=ab2) && (r->y+(r->h/2)<=ab4))
// DUAL select those in bounding box
					r->selection = TRUE;
					main->gui->monitor->select_action(r);
			}
			for (ln = main->session->data->machines.get_head(); ln!=NULL; ln=ln->next)
			{
				m = (Machine*) ln->item;
				if ((m->shown==TRUE) &&
						(m->x-(m->w/2)>=ab1) && (m->x+(m->w/2)<=ab3) &&
						(m->y-(m->h/2)>=ab2) && (m->y+(m->h/2)<=ab4))
					m->selection = TRUE;
			}
			for (ln = main->session->data->networks.get_head(); ln!=NULL; ln=ln->next)
			{
				n = (Network*) ln->item;
				if ((n->shown==TRUE) &&
						(n->x-(n->w/2)>=ab1) && (n->x+(n->w/2)<=ab3) &&
						(n->y-(n->h/2)>=ab2) && (n->y+(n->h/2)<=ab4))
					n->selection = TRUE;
			}
		}else
		if ((button == 1) && (mode == 1))
		{
			if (cur!=NULL)
			{
				if (cur->selection==FALSE)
				{
					// deselect everything else
					main->gui->monitor->select_all(FALSE);
					main->session->data->select(FALSE);
					// now select current object
					cur->selection = TRUE;
// DUAL
					if (cur->maintype==visual_device)
						main->gui->monitor->select_action((Device*)cur);
				}

				for (ln = main->session->data->devices.get_head(); ln!=NULL; ln=ln->next)
				{
					r = (Device*) ln->item;
//					r->fix = 0;
					if (r->selection==TRUE)
					{
						r->x = (long) (r->x+((x-nx)*scale));
						r->y = (long) (r->y+((y-ny)*scale));
					}
				}
				for (ln = main->session->data->machines.get_head(); ln!=NULL; ln=ln->next)
				{
					m = (Machine*) ln->item;
//					m->fix = 0;
					if (m->selection==TRUE)
					{
						m->x = (long) (m->x+((x-nx)*scale));
						m->y = (long) (m->y+((y-ny)*scale));
					}
				}
				for (ln = main->session->data->networks.get_head(); ln!=NULL; ln=ln->next)
				{
					n = (Network*) ln->item;
//					n->fix = 0;
					if (n->selection==TRUE)
					{
						n->x = (long) (n->x+((x-nx)*scale));
						n->y = (long) (n->y+((y-ny)*scale));
					}
				}

				// lower current object so shown last (on top)
				if (cur->maintype==visual_network)
					main->session->data->networks.lower(cur);
				else
				if (cur->maintype==visual_machine)
					main->session->data->machines.lower(cur);
				else
					main->session->data->devices.lower(cur);
			}
		}else
		if ((button == 1) && (mode == 2))
		{
			if (cur!=NULL)
			{
				if ((cur->shown==TRUE) &&
						(cur->x-(cur->w/2)<=orgx) && (cur->x+(cur->w/2)>=orgx) &&
						(cur->y-(cur->h/2)<=orgy) && (cur->y+(cur->h/2)>=orgy))
				{
// DUAL
					if (cur->selection==FALSE)
						cur->selection = TRUE;
					else
						cur->selection = FALSE;

					if (cur->maintype==visual_device)
						main->gui->monitor->select_action((Device*)cur);
				}
			}
		}

		mode = -1;
		gdk_pointer_ungrab(GDK_CURRENT_TIME);
		ready_to_merge = main->session->data->readyToMerge();
		main->gui->monitor->devicesidebar->mergeButton(ready_to_merge);
		main->gui->update_views();
		button = 0;
	}
}

void Viewer::draw_bb(GdkPixmap *w)
{
	gdk_draw_line(w, xgc, obx, oby, bbx, oby);
	gdk_draw_line(w, xgc, obx, oby, obx, bby);
	gdk_draw_line(w, xgc, bbx, oby, bbx, bby);
	gdk_draw_line(w, xgc, obx, bby, bbx, bby);
}

void Viewer::setScale(float z)
{
	float zm = (float((int(z*100))+1))/100;
	GTK_ADJUSTMENT(hscale3_adj)->lower = 1;
	GTK_ADJUSTMENT(hscale3_adj)->upper = zm;
	GTK_ADJUSTMENT(hscale3_adj)->step_increment = (float((zm-1)/100));
	GTK_ADJUSTMENT(hscale3_adj)->page_increment = (float((zm-1)/4));
	gtk_signal_emit_by_name (GTK_OBJECT (hscale3_adj), "changed");
}

void Viewer::fitToWindow()
{
	float t1, t2;

	main->session->data->sizeshow();

	oldxpos = xpos;
	oldypos = ypos;
	oldzoom = scale;
	t1 = float((main->session->data->width)/(float)(width-(BORDER*2)));
	t2 = float((main->session->data->height)/(float)(height-(BORDER*2)));
	newzoom = (t1>t2)?t1:t2;
	if (newzoom<1.0) newzoom=1.0; else newzoom=(float((int(newzoom*100))+1))/100;
	newxpos = (long) main->session->data->minx+(main->session->data->width/2);
	newypos = (long) main->session->data->miny+(main->session->data->height/2);
	if (animate_state == 0)
		animate_tag = gtk_timeout_add(30, animate_cb, this);
	animate_state = (animate_version==TRUE)?ANIMATE_STEPS:1;
}

void Viewer::fitToSelected()
{
	float t1, t2;

	if (main->session->data->selected_size()!=1) return;

	oldxpos = xpos;
	oldypos = ypos;
	oldzoom = scale;
	t1 = float((main->session->data->width)/(float)(width-(BORDER*2)));
	t2 = float((main->session->data->height)/(float)(height-(BORDER*2)));
	newzoom = (t1>t2)?t1:t2;
	if (newzoom<1.0) newzoom=1.0; else newzoom=(float((int(newzoom*100))+1))/100;
	newxpos = (long) main->session->data->minx+(main->session->data->width/2);
	newypos = (long) main->session->data->miny+(main->session->data->height/2);
	if (animate_state == 0)
		animate_tag = gtk_timeout_add(30, animate_cb, this);
	animate_state = (animate_version==TRUE)?ANIMATE_STEPS:1;
}

int Viewer::fly()
{
	float t1, t2;
	float sf;

	if (animate_state>0)
	{
		animate_state--;

		// line to produce linear motion
//		sf = (ANIMATE_STEPS-animate_state)/(float)ANIMATE_STEPS;

		// line to produce a parabolic motion
		sf = (cos(animate_state*M_PI/ANIMATE_STEPS)+1)/2;

		scale = oldzoom+((newzoom-oldzoom)*sf);
		xpos = (long) (oldxpos+((newxpos-oldxpos)*sf));
		ypos = (long) (oldypos+((newypos-oldypos)*sf));

		GTK_ADJUSTMENT(hscale3_adj)->value = scale;
		gtk_signal_emit_by_name (GTK_OBJECT (hscale3_adj), "changed");
		GTK_ADJUSTMENT(vscrollbar2_adj)->value = ypos;
		GTK_ADJUSTMENT(hscrollbar1_adj)->value = xpos;
		draw();
	}
	if (animate_state==0)
	{
		main->session->data->sizeshow();
		t1 = float((main->session->data->width)/(float)(width-(BORDER*2)));
		t2 = float((main->session->data->height)/(float)(height-(BORDER*2)));
		if ((t1<zoom) || (t2<zoom))
		{
			zoom = (t1>t2)?t1:t2;
			setScale(zoom);
		}
	}
	return (animate_state>0);
}

void Viewer::draw()
{
	float t1, t2;
	gint ow, oh;

	oh = height;
	ow = width;

	if (drawingarea1->window==NULL) return;
	if (main->session->data==NULL) return;

	width = drawingarea1->allocation.width;
	height = drawingarea1->allocation.height;

	if ((canvas == NULL) || (oh != height) || (ow != width))
	{
		if (canvas) gdk_pixmap_unref(canvas);
		canvas = gdk_pixmap_new(drawingarea1->window, width, height, -1);
	}

	main->session->data->sizeshow();
	t1 = float((main->session->data->width)/(float)(width-(BORDER*2)));
	t2 = float((main->session->data->height)/(float)(height-(BORDER*2)));
	if ((t1>zoom) || (t2>zoom) || (oh > height) || (ow > width))
	{
		zoom = (t1>t2)?t1:t2;
		setScale(zoom);
	}

	GTK_ADJUSTMENT(vscrollbar2_adj)->lower = 0;
	GTK_ADJUSTMENT(hscrollbar1_adj)->lower = 0;
	GTK_ADJUSTMENT(vscrollbar2_adj)->upper = main->session->data->height+(height*scale);
	GTK_ADJUSTMENT(hscrollbar1_adj)->upper = main->session->data->width+(width*scale);
	GTK_ADJUSTMENT(vscrollbar2_adj)->page_size = (height*scale);
	GTK_ADJUSTMENT(hscrollbar1_adj)->page_size = (width*scale);
	GTK_ADJUSTMENT(vscrollbar2_adj)->page_increment = (height*scale)/2;
	GTK_ADJUSTMENT(hscrollbar1_adj)->page_increment = (width*scale)/2;
	GTK_ADJUSTMENT(vscrollbar2_adj)->step_increment = (height*scale)/2;
	GTK_ADJUSTMENT(hscrollbar1_adj)->step_increment = (width*scale)/2;

	if (main->session->data->minx!=0) xpos-=main->session->data->minx;
	GTK_ADJUSTMENT(hscrollbar1_adj)->value = xpos;
	if (main->session->data->miny!=0) ypos-=main->session->data->miny;
	GTK_ADJUSTMENT(vscrollbar2_adj)->value = ypos;

	if (xpos>main->session->data->width) xpos = main->session->data->width;
	if (ypos>main->session->data->height) ypos = main->session->data->height;

	gtk_signal_emit_by_name (GTK_OBJECT (hscrollbar1_adj), "changed");
	gtk_signal_emit_by_name (GTK_OBJECT (vscrollbar2_adj), "changed");

	diagram->draw(canvas, main->session->data, scale, xpos, ypos, width, height, 0);
//	printf("xpos=%ld, ypos=%ld, diagw=%ld, diagh=%ld, width=%ld, height=%ld\n",
//		xpos, ypos, main->session->data->width, main->session->data->height,
//		width, height);

	// poor mans delay (for testing)
//	int h; for (h=0; h<100000000; h++);

	screw_up_dnd = 1;
	gdk_draw_pixmap(drawingarea1->window,
		drawingarea1->style->fg_gc[GTK_WIDGET_STATE(drawingarea1)],
		canvas, 0, 0, 0, 0, drawingarea1->allocation.width,
		drawingarea1->allocation.height);
}

