/* device.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <string.h>
#include <stdlib.h>
#include "device.h"
#include "network.h"
#include "interface.h"
#include "filter.h"
#include "subnet.h"
#include "misc.h"

Device::Device()
{
	empty();
}

Device::~Device()
{
	empty();
}

void Device::empty()
{
	Interface *in;
	MACAddress *m;
	Link *ln;
	int i;

	x = rand()%4000;
	y = rand()%3000;
	w = h = 192;
	selection = FALSE;
	shown = TRUE;
	maintype = visual_device;
	subtype = device_unknown;

	pingname.empty();
	pingname2.empty();
	strcpy(name, "");
	last_seen.empty();
	strcpy(location, "");
	strcpy(community, "");
	strcpy(community2, "");
	strcpy(descr, "");
	strcpy(comment, "");
	strcpy(contact, "");
	status = '0';
	group = device_group_bunch;
	grab_top = '0';
	grab_stats = '0';
	interfaces.empty();
//	merged = FALSE;
	tag = 0;
}

void Device::savexml(xmlNodePtr xmlnode)
{
	xmlNodePtr newxml;
	xmlNodePtr cur;
	gchar buf[16];
	Link *ln;
	Interface *in;
	MACAddress *m;

	newxml = xmlNewChild(xmlnode, NULL, (xmlChar*)"Device", NULL);
	if (!pingname.zero())
		xmlSetProp(newxml, (xmlChar*)"pingname", (xmlChar*)pingname.getstring());
	if (!pingname2.zero())
		xmlSetProp(newxml, (xmlChar*)"pingname2", (xmlChar*)pingname2.getstring());
	if (strcmp(name, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"name", (xmlChar*)name);
	if (!last_seen.zero())
		xmlSetProp(newxml, (xmlChar*)"lastseen", (xmlChar*)last_seen.getstring());
	if (strcmp(location, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"location", (xmlChar*)location);
	if (strcmp(descr, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"description", (xmlChar*)descr);
	if (strcmp(comment, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"comment", (xmlChar*)descr);
	if (strcmp(contact, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"contact", (xmlChar*)contact);
	if (strcmp(community, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"community", (xmlChar*)community);
	if (strcmp(community2, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"community2", (xmlChar*)community2);
	sprintf(buf, "%c", grab_top);
	xmlSetProp(newxml, (xmlChar*)"grab_topology", (xmlChar*)buf);
	sprintf(buf, "%c", grab_stats);
	xmlSetProp(newxml, (xmlChar*)"grab_stats", (xmlChar*)buf);
	strcpy(buf, deviceTypeToString(subtype));
	xmlSetProp(newxml, (xmlChar*)"subtype", (xmlChar*)buf);
	strcpy(buf, deviceGroupTypeToString(group));
	xmlSetProp(newxml, (xmlChar*)"group", (xmlChar*)buf);
	sprintf(buf, "%d", x);
	xmlSetProp(newxml, (xmlChar*)"x", (xmlChar*)buf);
	sprintf(buf, "%d", y);
	xmlSetProp(newxml, (xmlChar*)"y", (xmlChar*)buf);
	sprintf(buf, "%s", (shown==TRUE)?"yes":"no");
	xmlSetProp(newxml, (xmlChar*)"show", (xmlChar*)buf);

	for (ln=interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		if (in!=NULL)
			in->savexml(newxml);
	}
}

void Device::loadxml(xmlNodePtr xmlnode, List *ntwk, List *mch)
{
	xmlChar *buf;
	Interface *in;
	MACAddress *m;
	Network *n;

	empty();

	if (!xmlnode || !xmlnode->name || strcmp((char*)xmlnode->name, "Device")!=0)
		return;
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"pingname");
	if (buf!=NULL) { pingname.setstring((char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"pingname2");
	if (buf!=NULL) { pingname2.setstring((char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"name");
	if (buf!=NULL) { strcpy(name, (char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"lastseen");
	if (buf!=NULL) { last_seen.setstring((char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"location");
	if (buf!=NULL) { strcpy(location, (char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"description");
	if (buf!=NULL) { strcpy(descr, (char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"comment");
	if (buf!=NULL) { strcpy(comment, (char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"contact");
	if (buf!=NULL) { strcpy(contact, (char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"community");
	if (buf!=NULL) { strcpy(community, (char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"community2");
	if (buf!=NULL) { strcpy(community2, (char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"grab_topology");
	if (buf!=NULL) { grab_top = buf[0]; free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"grab_stats");
	if (buf!=NULL) { grab_stats = buf[0]; free(buf); }
	maintype = visual_device;
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"subtype");
	if (buf!=NULL) { subtype = stringToDeviceType((gchar*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"group");
	if (buf!=NULL) { group = stringToDeviceGroupType((gchar*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"x");
	if (buf!=NULL) { sscanf((char*)buf, "%d", &x); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"y");
	if (buf!=NULL) { sscanf((char*)buf, "%d", &y); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"show");
	if (buf!=NULL)
	{ shown = (strcmp((gchar*)buf, "yes")==0)?TRUE:FALSE; free(buf); }

	xmlnode = xmlnode->childs;
	while (xmlnode != NULL)
	{
		if (xmlnode->name!=NULL)
		{
			if (strcmp((char*)xmlnode->name, "Interface")==0)
			{
				in = new Interface();
				in->loadxml(xmlnode, mch);
				in->device = this;
				interfaces.tag(in);
				n = new Network();
				n->interfaces.tag(in);
				in->network = n;
				ntwk->tag(n);
			}
		}
		xmlnode = xmlnode->next;
	}
}

int compare_device_names(Link *a, Link *b)
{
	Device *r1 = (Device*) a->item;
	Device *r2 = (Device*) b->item;
	return (strcmp(r1->name, r2->name));
}

int compare_device_ips(Link *a, Link *b)
{
	Device *r1 = (Device*) a->item;
	Device *r2 = (Device*) b->item;
	return (r1->pingname.compare(r2->pingname));
}

int Device::match(char *str)
{
	char nm[16];

	if (strlen(str)==0) return 0;

	if ((strlen(name)>0) && (lower_contains(name, str))) return 1;
	if ((strlen(location)>0) && (lower_contains(location, str))) return 2;
	if ((strlen(community)>0) && (lower_contains(community, str))) return 3;
	if ((strlen(community2)>0) && (lower_contains(community2, str))) return 4;
	if ((strlen(descr)>0) && (lower_contains(descr, str))) return 5;
	if ((strlen(comment)>0) && (lower_contains(comment, str))) return 6;
	if ((strlen(contact)>0) && (lower_contains(contact, str))) return 7;

	sprintf(nm, "%s", pingname.getstring());
	if (lower_contains(nm, str)) return 8;
	sprintf(nm, "%s", pingname2.getstring());
	if (lower_contains(nm, str)) return 9;

	// also check lastseen to match on date (but not in quick search)

	return 0;
}

int Device::matchsubnet(Subnet *s)
{
	if (pingname.in_subnet(s->ip, s->mask)) return 1;
	if (pingname2.in_subnet(s->ip, s->mask)) return 2;

	return 0;
}

void Device::matchfilter(Filter *f)
{
/*
	Link *ln, *ln2;
	IPAddress *tip;
	Interface *in;

	if (f->default_action==action_deny)
		show='0';
	else
		show='1';

	if (f->default_action==action_deny)
	{
		if (f->match(pingname)!=0) { show='1'; return; }
	}else
	{
		if (f->match(pingname)==0) { show='0'; return; }				
	}
	for (ln=interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		for (ln2=in->ip_addresses.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			tip = (IPAddress*) ln2->item;
			if (f->default_action==action_deny)
			{
				if (f->match(*tip)!=0) { show='1'; return; }
			}else
			{
				if (f->match(*tip)==0) { show='0'; return; }				
			}
		}
	}
*/
}

