#include <stdlib.h>
#include <string.h>
#include "ipaddress.h"
#include "interface.h"
#include "network.h"
#include "subnet.h"
#include "machine.h"
#include "ref.h"
#include "misc.h"

Interface::Interface()
{
	empty();
}

Interface::~Interface()
{
	empty();
}

void Interface::empty()
{
	MACAddress *mc;
	IPAddress *ip;
	Machine *m;
	Link *ln;

	index = -1;
	mac.empty();
	speed = 0;
	type = 0;
	status = '0';
	duplex = 'u';
	portfast = 'u';
	strcpy(comment, "");
	strcpy(descr, "");

	ip_addresses.empty();
	refs.empty();
	nexthops.empty();
	machines.empty();
	network = NULL;
	device = NULL;
}

void Interface::print()
{
	printf("Interface index=%d, mac=%s, speed=%lu, type=%d, status=%c, descr=%s, comment=%s\n", index, mac.getstring(), speed, type, status, descr, comment);
}

void Interface::savexml(xmlNodePtr xmlnode)
{
	xmlNodePtr newxml;
	xmlNodePtr cur;
	IPAddress *tmpip;
	MACAddress *mac2;
	Machine *m;
	Ref *ref;
	Subnet *sn;
	char buf[256];
	Link *ln;

	newxml = xmlNewChild(xmlnode, NULL, (xmlChar*)"Interface", NULL);
	sprintf(buf, "%d", index);
	xmlSetProp(newxml, (xmlChar*)"index", (xmlChar*)buf);
	if (!mac.zero())
		xmlSetProp(newxml, (xmlChar*)"mac", (xmlChar*)mac.getstring());
	sprintf(buf, "%lu", speed);
	xmlSetProp(newxml, (xmlChar*)"speed", (xmlChar*)buf);
	sprintf(buf, "%d", type);
	xmlSetProp(newxml, (xmlChar*)"type", (xmlChar*)buf);
	sprintf(buf, "%c", status);
	xmlSetProp(newxml, (xmlChar*)"status", (xmlChar*)buf);
	if (duplex != 'u')
	{
		sprintf(buf, "%c", duplex);
		xmlSetProp(newxml, (xmlChar*)"duplex", (xmlChar*)buf);
	}
	if (portfast != 'u')
	{
		sprintf(buf, "%c", portfast);
		xmlSetProp(newxml, (xmlChar*)"portfast", (xmlChar*)buf);
	}
	if (strcmp(descr, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"description", (xmlChar*)descr);
	if (strcmp(comment, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"comment", (xmlChar*)comment);
	for (ln=ip_addresses.get_head(); ln!=NULL; ln=ln->next)
	{
		cur = xmlNewChild(newxml, NULL, (xmlChar*)"Address", NULL);
		tmpip = (IPAddress*) ln->item;
		if (tmpip!=NULL)
			xmlSetProp(cur, (xmlChar*)"ip", (xmlChar*)tmpip->getstring());
	}
	for (ln=nexthops.get_head(); ln!=NULL; ln=ln->next)
	{
		cur = xmlNewChild(newxml, NULL, (xmlChar*)"NextHop", NULL);
		tmpip = (IPAddress*) ln->item;
		if (tmpip!=NULL)
			xmlSetProp(cur, (xmlChar*)"ip", (xmlChar*)tmpip->getstring());
	}
	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		if (m!=NULL)
			m->savexml(newxml);
	}
/*
	for (ln=refs.get_head(); ln!=NULL; ln=ln->next)
	{
		ref = (Ref*) ln->item;
		if (ref!=NULL)
			ref->savexml(newxml);
	}
*/
}

void Interface::loadxml(xmlNodePtr xmlnode, List *mch)
{
	IPAddress *tmpip;
	xmlChar *buf;
	Machine *m;
	MACAddress *mac2;
	Subnet sn, *sn2;
	Ref *ref;
	Link *ln;

	// check this is the correct  node
	if (!xmlnode->name || strcmp((char*)xmlnode->name, "Interface")!=0)
		return;

	empty();

	// read properties
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"index");
	if (buf!=NULL) { sscanf((char*)buf, "%d", &index); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"mac");
	if (buf!=NULL) { mac.setstring((char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"speed");
	if (buf!=NULL) { sscanf((char*)buf, "%lu", &speed); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"type");
	if (buf!=NULL) { sscanf((char*)buf, "%d", &type); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"status");
	if (buf!=NULL) { status = buf[0]; free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"duplex");
	if (buf!=NULL) { duplex = buf[0]; free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"portfast");
	if (buf!=NULL) { portfast = buf[0]; free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"description");
	if (buf!=NULL) { strcpy(descr, (char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"comment");
	if (buf!=NULL) { strcpy(comment, (char*)buf); free(buf); }

	xmlnode = xmlnode->childs;
	while (xmlnode != NULL)
	{
		if (xmlnode->name!=NULL)
		{
			if (strcmp((char*)xmlnode->name, "Address")==0)
			{
				tmpip = new IPAddress();
				buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"ip");
				if (buf!=NULL) { tmpip->setstring((char*)buf); free(buf); }
				ip_addresses.tag(tmpip);
			}else
			if (strcmp((char*)xmlnode->name, "NextHop")==0)
			{
				tmpip = new IPAddress();
				buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"ip");
				if (buf!=NULL) { tmpip->setstring((char*)buf); free(buf); }
				nexthops.tag(tmpip);
			}else
			if (strcmp((char*)xmlnode->name, "Machine")==0)
			{
				m = new Machine();
				m->loadxml(xmlnode);
				m->interface = this;
				machines.tag(m);
				mch->tag(m);
			}else
			if (strcmp((char*)xmlnode->name, "Ref")==0)
			{
				ref = new Ref();
				ref->loadxml(xmlnode);
				refs.tag(ref);
			}
		}
		xmlnode = xmlnode->next;
	}
}

int Interface::match(char *str)
{
	char nm[16];
	Link *ln;
	IPAddress *ip;

	if (strlen(str)==0) return 0;

	if ((strlen(descr)>0) && (lower_contains(descr, str))) return 1;
	if ((strlen(comment)>0) && (lower_contains(comment, str))) return 2;

	sprintf(nm, "%s", mac.getstring());
	if (lower_contains(nm, str)) return 3;

	for (ln=ip_addresses.get_head(); ln!=NULL; ln=ln->next)
	{
		ip = (IPAddress*) ln->item;
		sprintf(nm, "%s", ip->getstring());
		if (lower_contains(nm, str)) return 4;
	}

	return 0;
}

int Interface::matchsubnet(Subnet *s)
{
	Link *ln;
	IPAddress *ip;

	for (ln=ip_addresses.get_head(); ln!=NULL; ln=ln->next)
	{
		ip = (IPAddress*) ln->item;
		if (ip->in_subnet(s->ip, s->mask)) return 1;
	}

	return 0;
}

